#
# $Header:   //ARCHIVE/Archive_NLS/scripts/SendMVSFileStatusReq.ksv   1.8.1.2   May 24 2001 14:02:34   Xiaoyan  $ 
#

delay_seconds=125;
bAutoGenNowMNRE=1;

if [ $bAutoGenNowMNRE -eq 1 ]; then;
  autogen='Auto-generate'
else
  autogen='Do not Auto-generate'
fi;

usage	()
{
  print ''
	print 'Usage: '$0' -c<cid> -i<id> [-n] [-d<delay_seconds>]'
  print ''
  print '  Required Param(s):'
  print '  -c<cid>                 Object''s ClassID'
  print '  -i<id>                  Object''s ID'
  print ''
  print '  Optional Param(s):'
  print '  -n                      Do NOT auto-generate MNRE statement for "NOW" if no FSRQ objects exist (default='$autogen')'
  print '  -d<delay_seconds>       Number of seconds before command is executed (default='$delay_seconds')'
  print ''
  print '  Sample:'
  print '    '$0' -cCOMP -i2'
}


# Process input parameters
while getopts "c:i:d:n?" opt; do
	case $opt in
	c)  cid=$OPTARG;;
	i)  id=$OPTARG;;
	d)  delay_seconds=$OPTARG;;
  n)  bAutoGenNowMNRE=0;;
	\?) usage
		  exit 16
		  ;;
	esac
done
shift $(($OPTIND - 1))

# Check for required params
typeset -i bMissingParam=0;
if [ "$cid" = "" ];         then echo 'ERROR: Missing required param -c<cid>'; bMissingParam=1; fi
if [ "$id" = "" ];          then echo 'ERROR: Missing required param -i<id>'; bMissingParam=1; fi
if  [ $bMissingParam -eq 1 ]; then
	usage
	exit 17
fi

generate_control_statements()
{
  cid=${1:?'ERROR: output_line() - Missing Param <cid>'}
  id=${2:?'ERROR: output_line() - Missing Param <id>'}
  typeset -i last_os_id=0;

  # create output filename(s)
  osql_output_file=$output_path'/MVSFileStatusReq.'$timestamp'_'$$.ISQL.OUT.TXT
  osql_temp_output_file=$output_path'/MVSFileStatusReq.'$timestamp'_'$$.ISQL.TEMP.OUT.TXT

  osql_cmd='asisp_uploadGetFCTTimes '$cid', '$id', 1, '$bAutoGenNowMNRE
  osql -S$db_server -U$db_user -P$db_password -Q"$osql_cmd" -u -w5000 -o$osql_output_file

  # convert unicode output into UTF8
  utf2ucs -r < $osql_output_file > $osql_temp_output_file
  mv $osql_temp_output_file $osql_output_file

  awk -F"|" -v timestamp=$timestamp -v master_control_statement=$master_control_statement -v pid=$$ -v output_path=$output_path '
    BEGIN {
      last_os_id=0;
      last_obj_id=0;
      fct_time_count=0;
      bOneNow=0;
      MAX_FCT_ENTRIES=15;
    }
    /DATA_ROW/ {
      split($4,os,/ /); split($7,job,/ /)
      os_id=$3; os_name=os[1]; obj_cid=$5; obj_id=$6; obj_name=job[1]; run_time=$8

      # check for new OS
      if (os_id != last_os_id) {
        print "*** NEW Operating System ["os_name"] ***";

        # if not first object
        if (last_os_id != 0)
          printf("~\n") >> output_file;

        # create output filename(s)
        output_file=sprintf("%s/MVSFileStatusReq.%s_%d.CS.%s.%d.TXT", output_path, timestamp, pid, os_name, os_id);
        print os_id"|"output_file >> master_control_statement

        last_os_id = os_id;
        last_obj_id = 0;
      }

      # check for new CICS
      if ( obj_id != last_obj_id ) {
        print "----- NEW CICS ["obj_name"="obj_id"] -----";

        # if not first object
        if (last_obj_id != 0)
          printf("~\n") >> output_file;

        printf("02\\14\\02%s\\03%s\\04%6.6lX%4.4lX", obj_cid, obj_name, obj_id, 8) >> output_file;
        last_obj_id = obj_id
        fct_time_count=0;
        bOneNow=0;
      }

      gsub(/ /, "", run_time);
      if (run_time=="00:00:00") {
        run_time="NOW"
        bOneNow=1;
      }
      if (++fct_time_count<MAX_FCT_ENTRIES)
        printf("\\26%s", run_time) >> output_file;
      else if (fct_time_count<=MAX_FCT_ENTRIES) {
        # output the last entry.  Add a 'NOW' instead
        # of the time if has never been previously added.
        if (bOneNow==0)
          printf("\\26NOW") >> output_file;
        else
          printf("\\26%s", run_time) >> output_file;
      }
    }

    END {
      if (last_obj_id != 0)
        printf("~\n") >> output_file;
    }
  ' $osql_output_file

  if [ -f "$osql_output_file" ]; then;
    rm "$osql_output_file";
  fi
}

main()
{
  timestamp=`date +"%Y%m%d%H%M%S"`

  # Get Database Server, User, Password from Registry
  eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

  # determine the output directory
  eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\Log" | awk '/Directory/ {gsub(/\\\\/, "/", $3); printf("output_path=%s\n",$3);}'`
  if [ "$output_path" = "" ]; then;
    if [ "$TEMP" != "" ]; then; output_path=`echo $TEMP | awk '{gsub(/\\\\/, "/"); print;}'`;
    elif [ "$TMP" != "" ]; then; output_path=`echo $TMP | awk '{gsub(/\\\\/, "/"); print;}'`;
    elif [ "$TMPDIR" != "" ]; then; output_path=`echo $TMPDIR | awk '{gsub(/\\\\/, "/"); print;}'`;
    fi;
  fi

  # Create Master list of control statement files
  master_control_statement=$output_path'/MVSFileStatusReq.'$timestamp'_'$$.MasterListOfControlStatements.TXT
  if [ -f "$master_control_statement" ]; then
    rm "$master_control_statement"
  fi

  # generate control statements
  generate_control_statements $cid $id
  if [ ! -f "$master_control_statement" ]; then
    echo $0': No work to do'
    return
  fi

  # For each OS, wrap the control statements in JCL and send via FTP
  cat "$master_control_statement" | 
  while read line;
  do
  {
    # identify the OS id and OS control statement file
    eval `echo $line | awk -F"|" '{printf("os_id=%lu;output_file=%s\n",$1,$2);}'`
    
    # create audit_msg file
    audit_msg_file="$output_file".AUDIT.MSG
    echo $cid'|'$id'|ASI_02\\14|File Status Request (02\\14)' > "$audit_msg_file";
    
    # perform the upload
    if [ $delay_seconds -eq 0 ]; then;
      Upload_WrapAndSendAsFTP -cOS -i$os_id -f$output_file
    else
      SOON $delay_seconds sh Upload_WrapAndSendAsFTP -cOS -i$os_id -f$output_file
    fi
  }
  done;
  if [ -f "$master_control_statement" ]; then
    rm "$master_control_statement"
  fi
}

main

