# ---------------------------------------------------------------------------
#
# $Header:   //ARCHIVE/Archive_NLS/scripts/SendMVSRMFRegistrations.ksv   1.1.1.4   Jun 11 2001 13:35:08   DAVID  $ 
#
# SendMVSRMFRegistrations.ksh -c<cid> -i<id> [options...]
# 
# The script creates the UPLOAD-formatted data to support RMF registration
# and sends the registration data to the Source/390 Pump
#
# ---------------------------------------------------------------------------

# default values
timestamp=`date +"%Y%m%d%H%M%S"`
delay_seconds=0;
bTrusted=0;
command_codes='OFF,ON'
bQuite=0;

# Get Database Server, User, Password FROM Registry
eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

usage()
{
  cat <<USAGE_DOC

Usage: $0 -c<cid> -i<id> [-S<db_server>] [-U<db_user>] [-P<db_password>]

The following options are supported:

Required Parameter(s):
  -c<cid>              - class ID of object for which the RMF registrations will be sent.
  -i<id>               - ID of object for which the RMF registrations will be sent.

Optional Parameter(s):
  -S<db_server>        - specifies server name where database resides.  Default=[$db_server].
  -U<db_user>          - specifies database user name.  Default=[$db_user].
  -P<db_password>      - specifies password for database user.  Default=[$db_password].
  -E                   - use trusted database connection. Use this instead of -U<db_username> and -P<db_password>
  -l<command_codes>    - comma separated list of command codes to denote register/unregister.  Default=[${command_codes}].
  -q                   - quiet mode.  No non-data output.

Command Codes:
  ON  - Register the RMF metric
  OFF - Unregister the RMF metric
USAGE_DOC
}

# process user options/overrides
while getopts "c:i:l:S:U:P:Eq?h" opt; do
  case $opt in
  c) cid=$(echo $OPTARG | awk '{print toupper($0);}');;
  l) command_codes=$(echo $OPTARG | awk '{print toupper($0);}');;
  i) id=$OPTARG;;
  S) db_server=$OPTARG;;
  U) db_user=$OPTARG;;
  P) db_password=$OPTARG;;
  E) bTrusted=1;;
  q) bQuite=1;;
  \?|h) usage; exit 1 ;;
  *) print >&2 Error with option $OPTOPT; usage; exit 1;;
  esac
done
shift $OPTIND-1

# check for required parameters
if [ -z "${cid}" ]; then
  echo "ERROR: missing required parameter -c<cid>." >&2
  usage
  exit 1
fi

# check for required parameters
if [ -z "${id}" ]; then
  echo "ERROR: missing required parameter -i<id>." >&2
  usage
  exit 1
fi

# check for OSQL utility
rc=`which osql.exe`; rc=$?
if [ $rc -ne 0 ]; then;
  echo "ERROR: Could not find OSQL.EXE.  Please install SQL Client Utilities." >&2
  exit 1
fi

# check for REGDMP utility
rc=`which REGDMP.exe`; rc=$?
if [ $rc -ne 0 ]; then;
  echo "ERROR: Could not find REGDMP.EXE.  Please install Windows NT Resource Kit Supplement 3" >&2
  echo "       or later and ensure REGDMP.EXE is in the path." >&2
  exit 1
fi

# determine OSQL command string
if [ $bTrusted -eq 1 ]; then
  SQLCMD="osql -S$db_server -E -n -w3000 -t120 -l120 -u"
else
  SQLCMD="osql -S$db_server -U$db_user -P$db_password -n -w3000 -t120 -l120 -u"
fi

get_rmf_data()
{
  sql_outfile="${1?"ERROR: get_rmf_data() missing required parameter <sql_outfile>."}"

  $SQLCMD -o"$sql_outfile" <<HERE_FILE
  USE Object
  SET NOCOUNT ON
  EXEC asisp_selectRMFProfile "${cid}", ${id}
HERE_FILE
}

process_rmf_data()
{
  on_off_flag="${1?"ERROR: process_rmf_data() missing required parameter <on_off_flag>."}"
  input_file="${2?"ERROR: process_rmf_data() missing required parameter <input_file>."}"

  # DATA_ROW|OS|6|1|IP01|@DELJOBS||IPO1|MVS_IMAGE|GT|0.000000|8D12D0|G||
  # DATA_ROW|OS|6|1|IP01|@NUMUSRS||IPO1|MVS_IMAGE|GT|0.000000|8D0D50|G||
  # DATA_ROW|OS|6|1|IP01|@PCTIDLE||IPO1|MVS_IMAGE|LT|100.000000|8D03E0|G||
  awk -F"|" -v on_off_flag=${on_off_flag} -v timestamp=$timestamp -v master_control_statement=$master_control_statement -v pid=$$ -v output_path=$output_path '
  BEGIN {
    os_id=0;
  }

  /DATA_ROW/ {
    cid=$2;
    cno=$3;
    id=$4;
    obj_name=$5;
    metric_basename=$6;
    param_01=$7;
    param_02=$8;
    param_03=$9;
    param_04=$10;
    param_05=$11;
    metric_comparison=$12;
    metric_value=$13;
    metric_id=$14;
    metric_category=$15;
    metric_description=$16;
    interval=int($17);
    alignment_time=int($18);

    if (os_id != id ) {
      print "*** NEW Operating System ["obj_name"] ***";
      close(output_file);

      split(obj_name,os_name,/ /);
      output_file=sprintf("%s/MVSRMFRegistrations.%s_%d.CS.%s.%d.TXT", output_path, timestamp, pid, os_name[1], id);
      print id"|"os_name[1]"|"output_file >> master_control_statement;

      os_id = id;
    }

    # 34 - interval in seconds (integer)
    # 35 - alignment time (integer - MMSS time passed the hour to begin the intervals)
    # param_04 (string)
    # param_05 (string)

    printf("02\\20\\02OS\\04%6.6d%4.4d\\47%s\\30AOPRMFCP\\31%s,%s,%s %s,%s,%s %s\\32AOPRMFRP\\33%s,%s\\34%ld\\35%04.4ld\\36%s~\n",
        id, cno, on_off_flag,
        param_01, param_02, param_03, param_04, param_05, metric_category, metric_id, 
        metric_comparison, metric_value, interval, alignment_time, metric_basename) >> output_file;
  }
  ' "${input_file}"
}

main()
{
  output_file="$0.sql.out.txt"
  rm -f "${output_file}" 2>nul:

  # determine the output directory
  eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\Log" | awk '/Directory/ {gsub(/\\\\/, "/", $3); printf("output_path=%s\n",$3);}'`
  if [ "$output_path" = "" ]; then;
    if [ "$TEMP" != "" ]; then; output_path=`echo $TEMP | awk '{gsub(/\\\\/, "/"); print;}'`;
    elif [ "$TMP" != "" ]; then; output_path=`echo $TMP | awk '{gsub(/\\\\/, "/"); print;}'`;
    elif [ "$TMPDIR" != "" ]; then; output_path=`echo $TMPDIR | awk '{gsub(/\\\\/, "/"); print;}'`;
    fi;
  fi

  # Create Master list of control statement files
  master_control_statement=$output_path'/MVSRMFRegistrations.'$timestamp'_'$$.MasterListOfControlStatements.TXT
  if [ -f "$master_control_statement" ]; then
    rm "$master_control_statement"
  fi

  # get the RMF registration data from the database
  get_rmf_data "${output_file}"

  # convert unicode output into UTF8
  output_temp_file="$0.sql.out.temp.txt"
  rm -f "${output_temp_file}" 2>nul:
  utf2ucs -r < $output_file > $output_temp_file
  mv $output_temp_file $output_file

  process_rmf_data COMMAND_CODE_HERE "${output_file}"

  # For each OS, wrap the control statements in JCL and send via FTP
  cat "$master_control_statement" | 
  while read line;
  do
  {
    # identify the OS id and OS control statement file
    eval `echo $line | awk -F"|" '{printf("os_id=%lu;os_name=%s;output_file=%s\n",$1,$2,$3);}'`

    # process each of the command codes
    rm -f "$0.foo.txt" 2>nul;
    for command_code in $(print "${command_codes}" | awk 'BEGIN {RS=","} {print}'); do
    {
      if [ $bQuite -eq 0 ]; then;
        print "Processing Command Code [${command_code}]..."
      fi

      case ${command_code} in
      ON) sed 's/COMMAND_CODE_HERE/ON/g' "${output_file}" >> "$0.foo.txt";;
      OFF) sed 's/COMMAND_CODE_HERE/OFF/g' "${output_file}" >> "$0.foo.txt";;
      *)  print >&2 "ERROR: invalid command code [${command_code}] in command code listing [${command_codes}]"; usage; exit 1;;
      esac
    }
    done;
    mv -vf "$0.foo.txt" "${output_file}"

    # create audit_msg file
    audit_msg_file="$output_file".AUDIT.MSG
    echo $cid'|'$id'|ASI_02\\20|RMF Registration (02\\20)' > "$audit_msg_file";
    
    # perform the upload
    if [ $delay_seconds -eq 0 ]; then;
      Upload_WrapAndSendAsFTP -cOS -i$os_id -f$output_file
    else
      SOON $delay_seconds sh Upload_WrapAndSendAsFTP -cOS -i$os_id -f$output_file
    fi
  }
  done;
  if [ -f "$master_control_statement" ]; then
    rm "$master_control_statement"
  fi
}

main
