# $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/imDeltaRestart.ksv   1.2   Jun 19 2001 14:06:04   lin  $
# $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/imDeltaRestart.ksv  $
# $Revision:   1.2  $
# $Date:   Jun 19 2001 14:06:04  $
# $Author:   lin  $
#!/bin/sh
#set -x
#

localhost="$(print "$COMPUTERNAME" | awk '{print toupper($0);}')"

INSTALLDIR=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g' | sed 's/\/$//g') 

if [ ! -d "$INSTALLDIR" ]; then
  print >&2 "Invalid installation directory: $INSTALLDIR"
  print >&2 "Tivoli Business Systems Manager is not installed on this system"
  exit 1
fi

eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

if [ -z $db_server ] || [ -z $db_user ] || [ -z $db_password ]; then
  print >&2 "TBSM SQL Server is not set up on this system"
  exit 1
fi

db_server=$(echo $db_server | awk '{print toupper($0);}')

if [ "$db_server" != "$localhost" ]; then
  print >&2 "Please run $0 on TBSM SQL Server $db_server"
  exit 1
fi

usage   ()
{
  print ''
  print 'Usage: '$0' [options...]'
  print ''
  print '  -S<db_server>           Specifies server name where database resides.  Default=['$db_server'].'
  print '  -U<db_user>             Specifies database user name.  Default=['$db_user'].'
  print '  -P<db_password>         Specifies password for database user.  Default=['$db_password'].'
  print '  -E                      Use trusted database connection. Use this instead of -U<username> and -P<password>'
  print ''
  print '  -A                      Remove all snapshot version'
  print '  -L                      Remove last snapshot version'
  print '  -C                      Remove everything for installation management'
  print ''
}

while getopts "U:S:P:ELCAh?" c; do
  case $c in
  U)  db_user=$OPTARG;;
  S)  db_server=$(echo $OPTARG | awk '{print toupper($0);}');;
  P)  db_password=$OPTARG;;
  E)  bTrusted=1;;
  L)  RemoveLast=1;;
  A)  RemoveAll=1;;
  C)  RemoveEverything=1;;
  h)  usage; exit 1;;
  \?) usage; exit 1;;
  *)  print >&2 Error with option $OPTOPT; usage; exit 1;;
  esac
done

#
if [ "$bTrusted" -eq 1 ]; then
  #SQLCMD="tbsmsql -S$db_server -E -n -w3000 "
  SQLCMD="osql -h-1 -S$db_server -E -n -w3000 "
else
  #SQLCMD="tbsmsql -S$db_server -U$db_user -P$db_password -n -w3000 "
  SQLCMD="osql -h-1 -S$db_server -U$db_user -P$db_password -n -w3000 "
fi

# now that we are on the right host

if [ "$RemoveEverything" -eq 1 ]; then
  sql_cmd="SET NOCOUNT ON
    DROP TABLE imObjectList 
    DROP VIEW imObjectList_v
    DROP VIEW imMostRecentVersion
    DROP TABLE imSnapshotHistory
    DROP TABLE imObjectList_current_DB
    DROP TABLE imObjectList_current_FS
    DROP TABLE imObjectSeed
    DROP TABLE imExclusionPattern
    EXEC msdb..sp_delete_job @job_name=N'Installation Management Snapshot'
  "
  $SQLCMD -d'Object' -Q"$sql_cmd"
  exit
fi

if [ "$RemoveLast" -eq 1 ]; then
  sql_cmd="SET NOCOUNT ON
    DECLARE @maxver INT
    SELECT @maxver=MAX(version) FROM imSnapshotHistory
    DELETE FROM imSnapshotHistory WHERE version=@maxver
    DELETE FROM imObjectList WHERE snapshotver=@maxver
  "
  $SQLCMD -d'Object' -Q"$sql_cmd"
  exit
fi

if [ "$RemoveAll" -eq 1 ]; then
  sql_cmd="SET NOCOUNT ON
    TRUNCATE TABLE imSnapshotHistory
    TRUNCATE TABLE imObjectList 
  "
  $SQLCMD -d'Object' -Q"$sql_cmd"
fi


