# $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/imDiffReport.ksv   1.0   Jun 18 2001 16:48:52   lin  $
# $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/imDiffReport.ksv  $
# $Revision:   1.0  $
# $Date:   Jun 18 2001 16:48:52  $
# $Author:   lin  $
#!/bin/sh
#set -x
#
# imDiffReport.ksh
#   TBSM Install Manager Database & File System objects change report script
#
#  Note:  
#
#

#
# Lookup the installation directory from the registry
#
INSTALLDIR=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g')

if [ ! -d "$INSTALLDIR" ]; then
  print >&2 "Invalid installation directory: $INSTALLDIR"
  print >&2 "Tivoli Business Systems Manager is not installed on this system"
  exit 1
fi

#
# check for OSQL utility
#
rc=`which osql.exe`; 
rc=$?
if [ $rc -ne 0 ]; then
  echo "ERROR: Could not find ISQL.EXE.  Please install SQL Client Utilities." >&2
  exit 1
fi

#
# Get Database Server, User, Password from Registry
#
eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

#
# set SQLCMD command
#
if [ "$bTrusted" -eq 1 ]; then
  #SQLCMD="tbsmsql -S$db_server -E -n -w3000 -dObject "
  SQLCMD="osql -S$db_server -E -n -w3000 -dObject "
else
  #SQLCMD="tbsmsql -S$db_server -U$db_user -P$db_password -n -w3000 -dObject "
  SQLCMD="osql -S$db_server -U$db_user -P$db_password -n -w3000 -dObject "
fi

getMAXver()
{
  sql_cmd="SET NOCOUNT ON; SELECT MAX(version) FROM imSnapshotHistory"
  $SQLCMD -d'Object' -Q"$sql_cmd" 2> nul: | awk '/^ *[0-9][0-9]*/ {ver=$1} END {print ver}'
}

getMINver()
{
  sql_cmd="SET NOCOUNT ON; SELECT MIN(version) FROM imSnapshotHistory"
  $SQLCMD -d'Object' -Q"$sql_cmd" 2> nul: | awk '/^ *[0-9][0-9]*/ {ver=$1} END {print ver}'
}

#
# default values & local variables
#
HOSTNAME=`hostname`
show_history=0
deliminator="	"	# this is a default separator character <TAB>, not space characters
DASHLINE="-------"

usage()
{
  print ''
  print 'Usage: '$0' [options...]'
  print 'Output is always sent to stdout.'
  print ''
  print '  -S<db_server>       Specifies server name where database resides.  Default=['$db_server'].'
  print '  -U<db_user>         Specifies database user name.  Default=['$db_user'].'
  print '  -P<db_password>     Specifies password for database user.  Default=['$db_password'].'
  print '  -E                  Use trusted database connection. Use this instead of -U<username> and -P<password>'
  print ''
  print '  -O[vopc]            Specifies the sort order of the report output'
  print '                        v sort by snapshot version '
  print '                        o sort by Object name '
  print '                        p sort by path '
  print '                        c sort by context '
  print '                      for example to sort by datetime within sorted Object name specify -Odo '
  print '  -H<host>            Specifies the hostname to be reported '
  print '  -d                  Specifies the field deliminator character of output, default to <TAB>'
  print '  -v                  Show snapshot history' 
  print '  -f<input_string> '
  print '  -l<input_string> '
  print 'Where:'
  print '  <input_string>      input string to be converted to a version number'
  print '                      input_string can be in the following format:'
  print '                        date in YYYY-MM-DD[ hh:mm:ss] '
  print '                          example: 2001-06-22 or "2001-06-29 12:23:34" '
  print '                        version in digit'
  print '                          example: 2 or 29  '
  print '                        string in <type>:<name>[:<build>]'
  print '                          example: "Regular:TBSM 1.5" or "Patch:Patch 1.5.22:V3.2.1"  '
  print '  -h                  help'
  print '  -?                  help'
  print ''
}

while getopts "S:U:P:EO:H:f:l:vd:h?" c; do
  case $c in
  U)  db_user=$OPTARG;;
  S)  db_server=$(echo $OPTARG | awk '{print toupper($0);}');;
  P)  db_password=$OPTARG;;
  E)  bTrusted=1;;
  O)  orderby=$OPTARG;;
  H)  whereclause=$OPTARG;;
  f)  firstString=$OPTARG;;
  l)  lastString=$OPTARG;;
  v)  show_history=1;;
  d)  deliminator=$OPTARG;;

  h) usage; exit 1;;
  \?) usage; exit 1;;
  *)  print >&2 Error with option $OPTOPT; usage; exit 1;;
  esac
done

if [ $show_history -eq 1 ]; then
  sql_cmd="SET NOCOUNT ON
    SELECT type, name, build, state, CONVERT(NVARCHAR(13), date, 107) as date, 
      CONVERT(NVARCHAR(8), date, 108) as time, version
    FROM imSnapshotHistory 
    ORDER BY version
  "
  $SQLCMD -s"$deliminator" -Q"$sql_cmd" | grep -v -e $DASHLINE 
  exit
fi

#
# validate firstver, lastver
#
if [ -z "$firstString" ]; then
  firstver=$(getMINver)
else
  firstver=$(imVersionLookup -i"$firstString")
  if [ -z "$firstver" ]; then
    print 'Invalid first input_string entered, please try again!'
    exit 2
  fi
fi

if [ -z "$lastString" ]; then
  lastver=$(getMAXver)
else
  lastver=$(imVersionLookup -i"$lastString")
  if [ -z "$lastver" ]; then
    print 'Invalid last input_string entered, please try again!'
    exit 2
  fi
fi

# swap the first and last if user specified two ver in wrong order
if [ $lastver -le $firstver ]; then
  tmpver=$lastver
  lastver=$firstver
  firstver=$tmpver
fi

# construct WHERE clause
if [ ! -z "$whereclause" ]; then
  whereclause=$(print "AND s.host = '$whereclause' ")
fi

# construct ORDER BY clause
if [ ! -z "$orderby" ]; then
  orderbyclause="ORDER BY "
  i=0
  until ord=${orderby:$i:1}; [ "$ord" == "" ]
  do
    if [ "$ord" == "v" ] || [ "$ord" == "V" ]; then
      orderbyclause=$(print $orderbyclause "s.snapshotver,")
    elif [ "$ord" == "o" ] || [ "$ord" == "O" ]; then
      orderbyclause=$(print $orderbyclause "s.name,")
    elif [ "$ord" == "p" ] || [ "$ord" == "P" ]; then
      orderbyclause=$(print $orderbyclause "s.path,")
    elif [ "$ord" == "c" ] || [ "$ord" == "C" ]; then
      orderbyclause=$(print $orderbyclause "s.context,")
    fi

    i=`expr $i + 1`
  done

  orderbyclause=$(echo $orderbyclause | awk '/,$/ {sub(",$", ""); print $0}')
fi


if [ -z "$orderby" ] && [ -z "$whereclause" ]; then
  sql_cmd="SET NOCOUNT ON
    EXEC sp_imObjectList_diff $firstver, $lastver"
else
  sql_cmd="SET NOCOUNT ON
    EXEC sp_imObjectList_diff $firstver, $lastver, \"$whereclause $orderbyclause\" "
fi

$SQLCMD -s"$deliminator" -Q"$sql_cmd" | grep -v -e $DASHLINE
