# $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/imSnapshotExtract.ksv   1.4   Jun 19 2001 14:38:02   lin  $
# $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/imSnapshotExtract.ksv  $
# $Revision:   1.4  $
# $Date:   Jun 19 2001 14:38:02  $
# $Author:   lin  $
#!/bin/sh
#set -x
#

localhost="$(print "$COMPUTERNAME" | awk '{print toupper($0);}')"

INSTALLDIR=$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g' | sed 's/\/$//g') 

if [ ! -d "$INSTALLDIR" ]; then
  print >&2 "Invalid installation directory: $INSTALLDIR"
  print >&2 "Tivoli Business Systems Manager is not installed on this system"
  exit 1
fi

eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

if [ -z $db_server ] || [ -z $db_user ] || [ -z $db_password ]; then
  print >&2 "TBSM SQL Server is not set up on this system"
  exit 1
fi

db_server=$(echo $db_server | awk '{print toupper($0)}')

if [ "$db_server" != "$localhost" ]; then
  print >&2 "Please run $0 on TBSM SQL Server $db_server"
  exit 1
fi

usage   ()
{
  print ''
  print 'Usage: '$0' [options...]'
  print ''
  print '  -S<db_server>           Specifies server name where database resides.  Default=['$db_server'].'
  print '  -U<db_user>             Specifies database user name.  Default=['$db_user'].'
  print '  -P<db_password>         Specifies password for database user.  Default=['$db_password'].'
  print '  -E                      Use trusted database connection. Use this instead of -U<username> and -P<password>'
  print '  -v<version>             Specifies snapshot version to extract, default to the last version'
  print ''
  print ''
}

while getopts "U:S:P:Ev:h?" c; do
  case $c in
  U)  db_user=$OPTARG;;
  S)  db_server=$(echo $OPTARG | awk '{print toupper($0);}');;
  P)  db_password=$OPTARG;;
  E)  bTrusted=1;;
  v)  ver=$OPTARG;;
  h)  usage; exit 1;;
  \?) usage; exit 1;;
  *)  print >&2 Error with option $OPTOPT; usage; exit 1;;
  esac
done

#
if [ "$bTrusted" -eq 1 ]; then
  #SQLCMD="tbsmsql -S$db_server -E -n -w3000 "
  SQLCMD="osql -h-1 -S$db_server -E -n -w3000 "
else
  #SQLCMD="tbsmsql -S$db_server -U$db_user -P$db_password -n -w3000 "
  SQLCMD="osql -h-1 -S$db_server -U$db_user -P$db_password -n -w3000 "
fi

#
# default to the last ver
#
if [ -z $ver ]; then
  ver=$($SQLCMD -d'Object' -Q"SET NOCOUNT ON; SELECT MAX(version) FROM imSnapshotHistory" 2> nul: | awk '/^ *[0-9][0-9]*/ {ver=$1} END {print ver}')
fi


sql_cmd="SET NOCOUNT ON; 
  SELECT type + '|' + name + '|' + build + '|<STATE>|<DATE>|<VERSION>'
  FROM imSnapshotHistory
  WHERE version = $ver
    AND state = 'Completed'
"
$SQLCMD -d'Object' -Q"$sql_cmd" 

sql_cmd="SET NOCOUNT ON; 
  SELECT DISTINCT '<HOST>|' + context + '|' + path + '|' + name + '|' + objectType + '|' +
    CAST(checksum AS NVARCHAR) + '|' +
    CAST(blockcount AS NVARCHAR) + '|' +
    CAST(base_schema_ver AS NVARCHAR) + '|' +
    CAST(schema_ver AS NVARCHAR) + '|' +
    CONVERT(NVARCHAR, crdate, 121) + '|' +
    CONVERT(NVARCHAR, mddate, 121) + '|' +
    '<SNAPSHOTVER>|0|' +
    CAST(deleted AS NVARCHAR)        
  FROM imObjectList 
  WHERE snapshotver = $ver
" 
$SQLCMD -d'Object' -Q"$sql_cmd" 


