# $Header:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/imSnapshotInsert.ksv   1.4   Jun 19 2001 14:40:10   lin  $
# $Archive:   //ARCHIVE/PVCSROOT/Archive_NLS/Scripts/imSnapshotInsert.ksv  $
# $Revision:   1.4  $
# $Date:   Jun 19 2001 14:40:10  $
# $Author:   lin  $
#!/bin/sh
#set -x
#
# imSnapshotInssert.ksh
#   TBSM Install Manager snapshot inserting script
#
#

#
# Lookup the installation directory from the registry
#
INSTALLDIR="$(regfind -p "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0" -n "InstallDirectory" 2>nul: | grep InstallDirectory | sed 's/[ ]*InstallDirectory = //' | sed 's?\\?/?g')"

if [ ! -d "$INSTALLDIR" ]; then
  print >&2 "Invalid installation directory: $INSTALLDIR"
  print >&2 "Tivoli Business Systems Manager is not installed on this system"
  exit 2
fi

#
# check for OSQL utility
#
rc=`which osql.exe`; 
rc=$?
if [ $rc -ne 0 ]; then
  echo "ERROR: Could not find ISQL.EXE.  Please install SQL Client Utilities." >&2
  exit 2
fi

localhost="$(print "$COMPUTERNAME" | awk '{print toupper($0);}')"

#
# Get Database Server, User, Password from Registry
#
eval `regdmp "HKEY_LOCAL_MACHINE\SOFTWARE\Accessible Software, Inc.\Access1\1.0\Settings\DB" | awk '/Server/ {printf("db_server=%s\n",$3);} /User/ {printf("db_user=%s\n",$3);} /Password/ {printf("db_password=%s\n",$3);}'`

db_server=$(echo $db_server | awk '{print toupper($0)}')

if [ "$db_server" != "$localhost" ]; then
  print >&2 "You are on $localhost. Please run $0 on TBSM SQL Server $db_server"
  exit 2
fi

#
# default values & local variables
#
bTrusted=0
currentDate="$(date '+%Y-%m-%d %H:%M:%S.000')"
batchsize=1000
DBINSERTFILE=$INSTALLDIR/Logs/imDB.insert


usage()
{
  print ''
  print 'Usage: '$0' -f<inputFile> [options...]'
  print ''
  print '  -S<db_server>           Specifies server name where database resides.  Default=['$db_server'].'
  print '  -U<db_user>             Specifies database user name.  Default=['$db_user'].'
  print '  -P<db_password>         Specifies password for database user.  Default=['$db_password'].'
  print '  -E                      Use trusted database connection. Use this instead of -U<username> and -P<password>'
  print ''
  print '  -f<inputFile>           Specifies the input file that contains snapshot and its history record'
  print ''
}

while getopts "U:S:P:T:Ef:h?" c; do
  case $c in
  U)  db_user=$OPTARG;;
  S)  db_server=$(echo $OPTARG | awk '{print toupper($0);}');;
  P)  db_password=$OPTARG;;
  E)  bTrusted=1;;
  f)  infile=$OPTARG;;
  h)  usage; exit 1;;
  \?) usage; exit 1;;
  *)  print >&2 Error with option $OPTOPT; usage; exit 1;;
  esac
done

if [ -z "$infile" ]; then
  print >&2 "No input file specified!"
  usage
  exit 2
fi

if [ ! -e "$infile" ]; then
  print >&2 "Input file $infile does not exist!"
  exit 2
fi

#
# set SQLCMD command
#
if [ "$bTrusted" -eq 1 ]; then
  #SQLCMD="tbsmsql -S$db_server -E -n -w3000 "
  SQLCMD="osql -h-1 -S$db_server -E -n -w3000 "
else
  #SQLCMD="tbsmsql -S$db_server -U$db_user -P$db_password -n -w3000 "
  SQLCMD="osql -h-1 -S$db_server -U$db_user -P$db_password -n -w3000 "
fi

snapshotRecord="$(head -1 $infile)"
snap_type=$(echo $snapshotRecord | awk -F"|" '{ print $1 }')
snap_name=$(echo $snapshotRecord | awk -F"|" '{ print $2 }')
snap_build=$(echo $snapshotRecord | awk -F"|" '{ print $3 }')

if [ -z "$snap_type" ]; then
  print >&2 "Unknown Snapshot Type $snap_type!"
  exit 3
fi

if [ -z "$snap_name" ]; then
  print >&2 "Unknown Snapshot Name $snap_name!"
  exit 3
fi
  
# get the snapshot version 
sql_cmd="SET NOCOUNT ON; SELECT MAX(version) FROM imSnapshotHistory"
snap_ver=$($SQLCMD -d'Object' -Q"$sql_cmd" 2> nul: | awk '/^ *[0-9][0-9]*/ {ver=$1} END {print ver}')

if [ -z "$snap_ver" ]; then
  snap_ver=0
fi

# increment the version 
snap_ver=`expr $snap_ver + 1`

sql_cmd="SET NOCOUNT ON
  INSERT INTO imSnapshotHistory 
  VALUES (N'$snap_type', N'$snap_name', N'$snap_build', 'Started', '$currentDate', $snap_ver) 
  "
$SQLCMD -d'Object' -Q"$sql_cmd" 

rm -rf $DBINSERTFILE

IFS="|"
tail +2 $infile | while read host context path name objectType checksum blockcount base_schema_ver schema_ver crdate mddate snapshotver confirmed deleted ; do
  if [ -z $host ]; then
    continue
  fi

  print "|$context|$path|$name|$objectType|$checksum|$blockcount|$base_schema_ver|$schema_ver|$crdate|$mddate|$snap_ver|0|$deleted" >> $DBINSERTFILE
#  print "$db_server|$context|$path|$name|$objectType|$checksum|$blockcount|$base_schema_ver|$schema_ver|$crdate|$mddate|$snap_ver|0|$deleted" >> $DBINSERTFILE

done

IFS=" "

# calculate the record count
recordCount=0
if [ -e $DBINSERTFILE ]; then
  recordCount=$(wc -l $DBINSERTFILE | awk '/^ *[0-9][0-9]* / {print $1}')
fi

print "  --Inserting $recordCount object(s) into imObjectList table on host $db_server. "
print "    Snapshot Type $snap_type, Name $snap_name, Build $snap_build, Version $snap_ver."

# check if we have any more work to do
if [ ! $recordCount -eq 0 ]; then
  sql_cmd="
    SET NOCOUNT ON
    BULK INSERT imObjectList FROM '$DBINSERTFILE'
    WITH (
    BATCHSIZE = $batchsize,
    DATAFILETYPE = 'char',
    FIELDTERMINATOR = '|',
    ROWTERMINATOR = '\n'
    )
  "
  $SQLCMD -d'Object' -Q"$sql_cmd" 
fi

# add a record with new timestamp and state of complete to the imSnapshotHistory table
now=$(date '+%Y-%m-%d %H:%M:%S.000')

sql_cmd="
  SET NOCOUNT ON
  INSERT INTO imSnapshotHistory 
  VALUES (N'$snap_type', N'$snap_name', N'$snap_build', 'Completed', '$now', $snap_ver) 
  "
$SQLCMD -d'Object' -Q"$sql_cmd" 

exit 

