-- $Header:   //Archive/Archive_NLS/DB/sql/DEFECT_77006.sqv   1.0   Jun 11 2001 12:08:22   Blaine  
--
-- DEFECT_77006.sql
--
-- Module Description
--   Fixed to _GetSummaryNotes for date and time.
--
USE Object
GO

---------------------------------------------------------------------------------------------------
-- DEFECT_77006.sql
-- $Archive:   //Archive/Archive_NLS/DB/sql/DEFECT_77006.sqv  
-- $Revision:   1.0  
-- $Date:   Jun 11 2001 12:08:22  
-- $Author:   Blaine  

PRINT 'Updating stored procedure  _GetSummaryNotes'
GO

PRINT 'Dropping stored procedure  _GetSummaryNotes'
DROP PROC _GetSummaryNotes
GO

PRINT 'Recreating stored procedure  _GetSummaryNotes'
GO

CREATE PROCEDURE _GetSummaryNotes
  @cid			            ClassID,
  @id			              ObjID,
  @note_type		        sysname = NULL,
  @note_status		      sysname = NULL,
  @note_owner		        sysname = NULL,
  @problem_status       sysname = NULL,
  @problem_assignedto   sysname = NULL  
AS
  RAISERROR('_GetSummaryNotes: cid=%s, id=%d, @note_type=%s, @note_status=%s, @note_owner=%s, @problem_status=%s, @problem_assignedto=%s', 0, 1, @cid, @id, @note_type, @note_status, @note_owner, @problem_status, @problem_assignedto)

  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _GetSummaryNotes
  ELSE
    SAVE TRAN _GetSummaryNotes

  DECLARE @filter_where NVARCHAR(255)

  DECLARE @sql NVARCHAR(255)
  DECLARE @APIHandle INT
  DECLARE @ErrorInfo INT
  DECLARE @lookup_ticket sysname
  DECLARE @ticket_status	NVARCHAR(60)
  DECLARE @ticket_description NVARCHAR(255)
  DECLARE @ticket_assignedto NVARCHAR(60)
  DECLARE @rc INT
  DECLARE @problem_system NVARCHAR(255)
  DECLARE @bHaveProblemSystem TINYINT

  IF EXISTS (SELECT * FROM SystemConfiguration WHERE groupname = 'PROBLEM_SYSTEM' AND propertyname = 'SYSTEM_NAME')
    BEGIN

      SELECT @bHaveProblemSystem = 1
      RAISERROR('_GetSummaryNotes: Have problem ticketing system integrated', 0, 1)
      SELECT @problem_system = propertyvalue FROM SystemConfiguration WHERE groupname = 'PROBLEM_SYSTEM' AND propertyname = 'SYSTEM_NAME'
      SET @problem_system = isnull(@problem_system,'')
      RAISERROR('_GetSummaryNotes: Inegrated Problem System problem_system=%s', 0, 1, @problem_system)
      
    END
  ELSE
    SELECT @bHaveProblemSystem = 0

  -- Temparary TABLE to hold final results.
  CREATE TABLE #notesummary
  (
    note_cid		          ClassID,
    note_id		            ObjID,
    note_owner		  sysname NULL,
    note_date		  NVARCHAR(30) NULL,
    note_time		  NVARCHAR(30) NULL,
    note_type		  sysname NULL,
    note_subject          NVARCHAR(255) NULL,
    note_status		  sysname NULL,
    note_description      NVARCHAR(255) NULL,
    note_age		  INT NULL,
    note_transferedto     NVARCHAR(255) NULL,
    note_problem_ticket   NVARCHAR(255) NULL,
    problem_status        sysname NULL,
    problem_description   NVARCHAR(255) NULL,
    problem_assignedto    NVARCHAR(255) NULL,
    note_datetime         DATETIME NULL
  )

  -- CONVERT LOB to the physical object
  IF RTRIM(@cid) = 'LOB'
    -- WHEN calling asisp_followphysicallink @parentcid must be non-NULL IF the RETURN parameters are to be filled IN
    EXEC asisp_followphysicallink @id, @cid OUTPUT, @id OUTPUT

  CREATE TABLE #notes_temp
  (
	  link_type 		    NVARCHAR(30) NULL,
	  src_cid 			    ClassID NULL,
	  src_cno  			    ObjID NULL,
	  src_id 			      ObjID NULL,
	  dst_cid 			    ClassID NULL,
	  dst_cno 			    INT NULL,
	  dst_id  			    ObjID NULL,
	  flags 			      INT NULL,
	  generation 		    INT NULL,
	  id 				        INT NULL,
	  name 				      NVARCHAR(100) NULL,
	  revision 			    INT NULL,
	  nchild 			      INT NULL,
	  description 		  NVARCHAR(255) NULL,
	  ctime 			      DATETIME NULL,
	  mtime 			      DATETIME NULL,
	  userid 			      NVARCHAR(30) NULL,
	  deleted 			    INT NULL,
	  _Date 			      DATETIME NULL,
	  _Time 			      DATETIME  NULL,
	  _UserID 			    NVARCHAR(30) NULL,
	  _Type 			      NVARCHAR(30) NULL,
	  _Status 			    NVARCHAR(30) NULL,
	  _AlertState 		  NVARCHAR(30) NULL,
	  _Subject 			    NVARCHAR(255) NULL,
    LBody             INT NULL,
	  _Body 			      NVARCHAR(255) NULL,
	  _ProblemID 		    sysname   NULL,
	  _CreatorObjID 	  ObjID NULL,
	  _CreatorCID 		  ClassID NULL,
	  _DurationMinutes 	INT NULL,
	  _ExpirationTime 	DATETIME NULL
	)

  INSERT INTO #notes_temp EXEC asisp_resolvelinkvattribute @cid, @id, 'Notes'

  SET @filter_where = ''
  IF ISNULL(@note_status,'') <> ''
    SELECT @filter_where = @filter_where + ' AND note_status = ''' + @note_status + ''''
  IF ISNULL(@note_type,'') <> ''
    SELECT @filter_where = @filter_where + ' AND note_type = ''' + @note_type + ''''
  IF ISNULL(@note_owner,'') <> ''
    SELECT @filter_where = @filter_where + ' AND note_owner = ''' + @note_owner + ''''
  IF ISNULL(@problem_status,'') <> ''
    SELECT @filter_where = @filter_where + ' AND problem_status = ''' + @problem_status + ''''
  IF ISNULL(@problem_assignedto,'') <> ''
    SELECT @filter_where = @filter_where + ' AND problem_assignedto = ''' + @problem_assignedto + ''''

  /*
  INSERT #notesummary 
     SELECT 'NOTE', id, _UserID, CONVERT(NVARCHAR(30),_Date,101), CONVERT(NVARCHAR(30),_Time,108), 
            _Type, _Status, _Body, DATEDIFF(mi,_Date,GETDATE()), _UserID, _ProblemID ,'' ,'' , '', _Date
       FROM #notes_temp
  */

  /* DEFECT_77006 change to use ctime for Date and Time columns of Notes View. */
  INSERT #notesummary 
     SELECT 'NOTE', id, _UserID, CONVERT(NVARCHAR(30),ctime,101), CONVERT(NVARCHAR(30),ctime,108), 
            _Type, _Subject, _Status, _Body, DATEDIFF(mi,ctime,GETDATE()), _UserID, _ProblemID ,'' ,'' , '', ctime
       FROM #notes_temp

  -- Lookup translated labels for Type and Status.
  UPDATE #notesummary
   SET note_type = ISNULL(ev.label,'Unknown')
   FROM enumeration e, enum_value ev 
   WHERE e.name = 'NoteTypes'
   AND e.eid = ev.eid
   AND #notesummary.note_type = ev.name

  UPDATE #notesummary
    SET note_status = ISNULL(ev.label,'Unknown')
    FROM enumeration e, enum_value ev 
    WHERE e.name = 'NoteStatus'
    AND e.eid = ev.eid
    AND #notesummary.note_status = ev.name

  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN _GetSummaryNotes

  -- Attempt to get extra problem ticket information only if any of the notes have a linked ticket.
  IF Exists(SELECT note_problem_ticket FROM #notesummary WHERE note_problem_ticket <> ""  AND note_problem_ticket <> "''")
  BEGIN

  -- Only implemented for TSD disttributed.
  IF @bHaveProblemSystem = 1 and @problem_system = 'TSD'
  BEGIN
    RAISERROR('_GetSummaryNotes: connecting to TSD...', 0, 1)
    EXEC _TSDAutoConnect 'ENT',1, @APIHandle OUTPUT, @ErrorInfo OUTPUT
  END
  IF @bHaveProblemSystem = 1 AND @ErrorInfo = 0
  BEGIN
    RAISERROR('_GetSummaryNotes: _TSDAutoConnect succeeded; APIHandle=%d.', 0, 1, @APIHandle)
    CREATE TABLE #ticket
    (
      ProblemId     NVARCHAR(255),
      XUser         NVARCHAR(255),
      XSystem       NVARCHAR(255),
      Component     NVARCHAR(255),
      Item          NVARCHAR(255),
      Module        NVARCHAR(255),
      ProblemType   NVARCHAR(255),
      ProblemCode   NVARCHAR(255),
      Severity      INT,
      DateEntered   NVARCHAR(255),
      DateUPDATEd   NVARCHAR(255),
      Description   NVARCHAR(255)
    )
	
    DECLARE  notesummary_cursor CURSOR FOR SELECT note_problem_ticket FROM #notesummary WHERE note_problem_ticket <> ""  AND note_problem_ticket <> "''"
    OPEN notesummary_cursor
    FETCH NEXT FROM notesummary_cursor INTO @lookup_ticket
    WHILE @@FETCH_STATUS = 0
    BEGIN
      INSERT INTO #ticket EXEC _TSDProblemList 'ENT',1,@APIHandle,  @lookup_ticket

      IF @@ROWCOUNT <> 0 
      BEGIN
        SELECT @ticket_status = ProblemCode, @ticket_description = Description, @ticket_assignedto = XUser
          FROM #ticket

        UPDATE #notesummary 
          SET problem_status = @ticket_status, 
              problem_description = @ticket_description, 
              problem_assignedto = @ticket_assignedto
          WHERE CURRENT OF notesummary_cursor
      END

      DELETE #ticket

      FETCH NEXT FROM notesummary_cursor INTO @lookup_ticket
    END
    CLOSE notesummary_cursor
    DEALLOCATE notesummary_cursor

    RAISERROR('_GetSummaryNotes: disconnecting FROM TSD using _TSDDisconnect...', 0, 1)
    EXEC @rc=_TSDDisconnect @cid, @id, @APIHandle
    IF @rc<0
    BEGIN
      RAISERROR('_GetSummaryNotes: ERROR: failed to disconnect FROM TSD.  rc=%d', 11, 1, @rc)
      --ROLLBACK TRAN _GetSummaryNotes
      RETURN -30000
    END
    ELSE
      RAISERROR('_GetSummaryNotes: disconnected successfully FROM TSD.', 0, 1)

    DROP TABLE #ticket

  END

  END

  IF ISNULL(@filter_where,"") <> ""
  BEGIN
    -- Remove leading AND FOR WHERE clause
    RAISERROR('_GetSummaryNotes: @filter_where clause=[%s]', 0, 1, @filter_where)
    
    SELECT @filter_where = ' WHERE ' + stuff(@filter_where,1,5,'')
    EXEC('SELECT * FROM #notesummary ' + @filter_where + ' ORDER BY note_datetime DESC') 
  END
  ELSE
    SELECT * 
      FROM #notesummary
      ORDER BY note_datetime DESC

  DROP TABLE #notes_temp
  DROP TABLE #notesummary

  -- Finish up TRANSACTION
  --IF @tcount = 0
    --COMMIT TRAN _GetSummaryNotes
GO

PRINT 'Stored Procedure  _GetSummaryNotes updated.'
GO