-- $Header:   //ARCHIVE/Archive_NLS/DB/SQL/DEFECT_77913.sqv   1.1   Jun 21 2001 10:39:14   BRENT  $
--
-- DEFECT_77913.sql
--
-- Module Description
--   Fix for defect 77913: RODM Discovery Process fails due to field truncation.
--
-- This happens when the data being processed contains values that are
-- longer than the name field for the objects. It is treated as an
-- error when the procedure is running as a scheduled task because the
-- database connection used by the SQLServerAgent has ANSI_WARNING
-- behavior turned on.
-- 
-- The solution is to modify the task command to explicitly turn off
-- the ANSI_DEFAULTS, ANSI_WARNINGS, and ANSI_NULLS settings.


USE msdb
GO


DECLARE @jobname NVARCHAR(100)
SET @jobname = 'RODM Discovery Process'

DECLARE @command NVARCHAR(256)
SET @command = 'SET ANSI_DEFAULTS OFF
SET ANSI_NULLS OFF
SET ANSI_WARNINGS OFF
EXEC _DiscoverRODMObjects'

UPDATE s SET
  command = @command
FROM msdb..sysjobs AS j 
  JOIN msdb..sysjobsteps AS s ON s.job_id = j.job_id
WHERE j.name = @jobname
  AND s.command <> @command

IF NOT EXISTS
(
  SELECT * FROM msdb..sysjobs AS j 
    JOIN msdb..sysjobsteps AS s ON s.job_id = j.job_id
  WHERE j.name = @jobname
    AND s.command = @command
)
  RAISERROR('ERROR: Could not find job ''%s'' or other error in update', 11, 1, @jobname)
ELSE
  PRINT 'Job ''' + @jobname + ''' successfully updated'
