-- $Header:   //ARCHIVE/Archive_NLS/db/sql/_SetMNO2Flag.sqv   1.1   Jun 18 2001 15:45:48   Marc  $

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_SetMNO2Flag' AND type = 'P')
  DROP PROCEDURE _SetMNO2Flag
GO
PRINT 'Creating procedure _SetMNO2Flag...'
GO

-- $Archive:   //ARCHIVE/Archive_NLS/db/sql/_SetMNO2Flag.sqv  $
-- $Author:   Marc  $
-- $Date:   Jun 18 2001 15:45:48  $
-- $Revision:   1.1  $
CREATE PROCEDURE _SetMNO2Flag
  @cid		ClassID,
  @id		ObjID,
  @field	NVARCHAR(100),
  @value	BIT
AS

  -- Accepted values for @field:
  -- Problem         0x00000001
  -- Change          0x00000002
  -- Automation      0x00000004

  -- standard transaction logic
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _SetMNO2Flag
  ELSE
    SAVE TRAN _SetMNO2Flag

  -- make sure the input params are correct
  IF @field IS NULL OR @value IS NULL
  BEGIN
    RAISERROR('The parameters field and value cannot be NULL.', 11, 1)
    ROLLBACK TRAN _SetMNO2Flag
    RETURN
  END

  -- find the old value of the flag
  CREATE TABLE #tempval ( flag INT )
  DECLARE @id_str NVARCHAR(10), @cname NVARCHAR(24), @table sysname, @flag_consolidated BIT, @oldval INT
  SET @id_str = CAST(@id AS NVARCHAR(10))
  SELECT @cname = cname, @flag_consolidated = flag_consolidated FROM obj_class WHERE cid = @cid
  IF @flag_consolidated = 1
    SET @table = @cname + '_C'
  ELSE
    SET @table = @cname + '_A'
  EXEC("INSERT INTO #tempval SELECT _flags FROM " + @table + " WHERE id = " + @id_str)
  SELECT @oldval = flag FROM #tempval

  -- Lookup the value from the enumeration
  DECLARE @bitval INT
  SELECT @bitval = value
  FROM enumeration e JOIN enum_value ev ON e.eid = ev.eid
  WHERE e.name = 'MNO2FlagValues' AND ev.name = @field
  IF @@ROWCOUNT = 0
  BEGIN
    RAISERROR('_SetMNO2Flag: Setting of field %s is not supported.', 11, 1, @field)
    ROLLBACK TRAN _SetMNO2Flag
    RETURN
  END

  -- find out if the field is a multi-value field
  DECLARE @num_me_values INT
  SELECT @num_me_values = COUNT(*)
  FROM enum_value ev JOIN enumeration e ON e.eid = ev.eid
  WHERE e.name = 'MNO2FlagValues'
    AND ((~ev.value) & @bitval) = 0
    AND ev.value <> @bitval
  IF (SELECT COUNT(*) FROM enum_value ev JOIN enumeration e ON e.eid = ev.eid WHERE e.name = 'MNO2FlagValues' AND (ev.value & @bitval) <> 0 AND ev.value <> @bitval) > @num_me_values
  BEGIN
    RAISERROR('_SetMNO2Flag: Setting of multi-value field %s is not supported.', 11, 1, @field)
    ROLLBACK TRAN _SetMNO2Flag
    RETURN
  END

  -- find any mutually exclusive values present in the enumeration
  DECLARE @me_value INT
  DECLARE cursor_ME INSENSITIVE CURSOR FOR
    SELECT ev.value
    FROM enum_value ev JOIN enumeration e ON e.eid = ev.eid
    WHERE e.name = 'MNO2FlagValues'
      AND ((~ev.value) & @bitval) = 0
      AND ev.value <> @bitval
  OPEN cursor_ME
  FETCH NEXT FROM cursor_ME INTO @me_value
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @oldval = @oldval & ~@me_value
    FETCH NEXT FROM cursor_ME INTO @me_value
  END
  CLOSE cursor_ME
  DEALLOCATE cursor_ME

  -- compute the new value of the attribute
  DECLARE @newval INT
  IF @value = 1
    SET @newval = @oldval | @bitval
  ELSE
    SET @newval = @oldval & ~@bitval

  -- set the new value
  DECLARE @newval_str NVARCHAR(255)
  SET @newval_str = CAST(@newval AS NVARCHAR(255))
  EXEC asisp_updateattr @cid, @id, 'flags', @newval_str

  -- send a notification
  EXEC asisp_notifyObjAttrUpdated @cid, @id, @newval_str, "flags"

  -- finish up transaction
  IF @tcount = 0
    COMMIT TRAN _SetMNO2Flag

GO

PRINT 'Done.'