-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/asisp_addDerivedMSGD.sqv   1.1   Jun 19 2001 13:09:08   Marc  $

USE Object
GO

PRINT 'Creating Procedure asisp_addDerivedMSGD...'

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_addDerivedMSGD' AND type = 'P')
  DROP PROCEDURE asisp_addDerivedMSGD
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_addDerivedMSGD.sqv  $
-- $Revision:   1.1  $
-- $Date:   Jun 19 2001 13:09:08  $
-- $Author:   Marc  $
CREATE PROCEDURE asisp_addDerivedMSGD
  @cid         ClassID,       -- the ClassID of the MessageDescription
  @MsgID       NVARCHAR(255), -- the MsgID of the MessageDescription 
  @dst_cid     ClassID,       -- the cid of the class to get the message description
  @hierarchy   INT = 0        -- indicates if all of the derived classes of @dst_cid are affected
AS

  -- This procedure creates message descriptions and state types for all derived classes on param @cid
  SET NOCOUNT ON

  -- Standard transaction logic
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_addDerivedMSGD
  ELSE
    SAVE TRAN asisp_addDerivedMSGD

  -- verify that the message description exists
  IF NOT EXISTS (SELECT * FROM MessageDescription_T WHERE obj_cid = @cid AND MsgID = @MsgID)
  BEGIN
    RAISERROR('asisp_addDerivedMSGD: ERROR: Message %s is not defined for %s.', 11, 1, @MsgID, @cid)
    ROLLBACK TRAN asisp_addDerivedMSGD
    RETURN
  END

  -- verify specified class is not abstract
  IF @hierarchy = 0 AND (SELECT flag_abstract FROM obj_class WHERE cid = @dst_cid) = 1
  BEGIN
    RAISERROR('asisp_addDerivedMSGD: ERROR: Class %s is abstract.  Specify @hierarchy=1 to add message to derived classes.', 11, 1, @dst_cid)
    ROLLBACK TRAN asisp_addDerivedMSGD
    RETURN
  END

  -- lookup the message
  DECLARE @Description NVARCHAR(255), @State NVARCHAR(40), @AlertState NVARCHAR(40), @Priority NVARCHAR(40), @Flags INT, @SE NVARCHAR(255), @CLP NVARCHAR(255), @PP NVARCHAR(255)
  SELECT @Description = Description, @State = State, @AlertState = AlertState, @Priority = Priority, @Flags = Flags,
    @SE = SearchExpression, @CLP = ChildLookupPattern, @PP = ProcessingPattern
  FROM MessageDescription_T WHERE obj_cid = @cid AND MsgID = @MsgID

  DECLARE @derived_cid ClassID, @gen INT

  IF @hierarchy <> 0
  BEGIN
    -- derived classes cursor
    DECLARE cursor_derived INSENSITIVE CURSOR FOR
      SELECT DISTINCT oc.cid, ic.generation
      FROM isa_chain ic JOIN obj_class oc ON oc.cid = ic.cid
      WHERE oc.flag_abstract = 0 AND ic.base_cid = @dst_cid
      ORDER BY ic.generation

    OPEN cursor_derived
    FETCH NEXT FROM cursor_derived INTO @derived_cid, @gen
    WHILE @@FETCH_STATUS <> -1
    BEGIN

      IF @derived_cid = @cid
      BEGIN
        FETCH NEXT FROM cursor_derived INTO @derived_cid, @gen
        CONTINUE
      END

      PRINT 'Defining message description ' + @MsgID + ' for class ' + @derived_cid + '...'
      EXEC asisp_defineMessageDescription @derived_cid, @MsgID, @Description, @State, @AlertState, @Priority, @Flags, 0, 	@SE, @CLP, @PP

      FETCH NEXT FROM cursor_derived INTO @derived_cid, @gen
    END
    CLOSE cursor_derived
    DEALLOCATE cursor_derived
  END
  ELSE
  BEGIN
    PRINT 'Defining message description ' + @MsgID + ' for class ' + @dst_cid + '...'
    EXEC asisp_defineMessageDescription @dst_cid, @MsgID, @Description, @State, @AlertState, @Priority, @Flags, 0, @SE, @CLP, @PP
  END

  -- Finish transaction
  IF @tcount = 0
    COMMIT TRAN asisp_addDerivedMSGD

GO

PRINT 'Done.'
GO