--
-- $Header:   //ARCHIVE/Archive_NLS/DB/sql/asisp_createRMFProfileDef.sqv   1.0   Jun 11 2001 13:31:32   DAVID  $
--

USE Object
GO
SET NOCOUNT ON
GO

-------------------------------------------------------------------------------
-- asisp_createRMFProfileDefEx
PRINT 'Creating PROCEDURE asisp_createRMFProfileDefEx...'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='asisp_createRMFProfileDefEx')
  DROP PROCEDURE asisp_createRMFProfileDefEx
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_createRMFProfileDef.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jun 11 2001 13:31:32  $
-- $Author:   DAVID  $
CREATE PROCEDURE asisp_createRMFProfileDefEx
  @profile_id           INT,
  @metric_basename      NVARCHAR(255),
  @param_01             NVARCHAR(50),
  @param_02             NVARCHAR(50),
  @param_03             NVARCHAR(50),
  @param_04             NVARCHAR(50),
  @param_05             NVARCHAR(50),
  @metric_category      NVARCHAR(1),
  @metric_id            NVARCHAR(50),
  @metric_comparison    NVARCHAR(2),
  @metric_value         DECIMAL(19,6),
  @metric_description   NVARCHAR(255) = NULL,
  @interval             INT = 100,
  @interval_alignment   INT = 0,
  @bEnabled             TINYINT = 1
AS
  RAISERROR('asisp_createRMFProfileDefEx: profile_id=%d, metric_basename=%s (%s,%s,%s) (%s,%s,%s), enabled=%d', 0, 1, 
              @profile_id, @metric_basename, 
              @param_01, @param_02, @param_03, 
              @param_04, @param_05, @metric_category, 
              @bEnabled)

  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount > 0
    SAVE TRANSACTION asisp_createRMFProfileDefEx
  ELSE
    BEGIN TRANSACTION asisp_createRMFProfileDefEx

  IF NOT EXISTS (SELECT * FROM RMFProfileDef WHERE id = @profile_id AND metric_basename = @metric_basename )
  BEGIN
    RAISERROR('asisp_createRMFProfileDefEx: inserting RMF Definition [%s] INTO Profile %d...', 0, 1, @metric_basename, @profile_id)
    INSERT RMFProfileDef(id, metric_basename, param_01, param_02, param_03, param_04, param_05, metric_comparison, metric_value, metric_id, metric_category, metric_description, enabled, interval, alignment)
      SELECT @profile_id, @metric_basename, @param_01, @param_02, @param_03, @param_04, @param_05, @metric_comparison, @metric_value, @metric_id, @metric_category, @metric_description, @bEnabled, @interval, @interval_alignment
  END
  ELSE
  BEGIN
    RAISERROR('asisp_createRMFProfileDefEx: updating RMF Definition [%s] IN Profile %d...', 0, 1, @metric_basename, @profile_id)
    UPDATE RMFProfileDef
      SET param_01=@param_01, param_02=@param_02, param_03=@param_03, param_04=@param_04, param_05=@param_05, 
          metric_comparison=@metric_comparison, metric_value=@metric_value, 
          metric_id=@metric_id, metric_category=@metric_category, metric_description=@metric_description, enabled=@bEnabled,
          interval=@interval, alignment=@interval_alignment
      WHERE id = @profile_id AND metric_basename = @metric_basename
  END

  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRANSACTION asisp_createRMFProfileDefEx
GO

-------------------------------------------------------------------------------
-- asisp_createRMFProfileDef
PRINT 'Creating PROCEDURE asisp_createRMFProfileDef...'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='asisp_createRMFProfileDef')
  DROP PROCEDURE asisp_createRMFProfileDef
GO

-- $Archive:   //ARCHIVE/Archive_NLS/DB/sql/asisp_createRMFProfileDef.sqv  $
-- $Revision:   1.0  $
-- $Date:   Jun 11 2001 13:31:32  $
-- $Author:   DAVID  $
CREATE PROCEDURE asisp_createRMFProfileDef
  @profile_id           INT,
  @metric_basename      NVARCHAR(255),
  @param_01             NVARCHAR(50),
  @param_02             NVARCHAR(50),
  @param_03             NVARCHAR(50),
  @metric_comparison    NVARCHAR(2),
  @metric_value         DECIMAL(19,6),
  @metric_id            NVARCHAR(50),
  @metric_category      NVARCHAR(1),
  @metric_description   NVARCHAR(255) = NULL,
  @bEnabled             TINYINT = 1
AS
  RAISERROR('asisp_createRMFProfileDef: profile_id=%d, metric_basename=%s (%s,%s,%s), enabled=%d', 0, 1, @profile_id, @metric_basename, @param_01, @param_03, @param_03, @bEnabled)
  DECLARE @rc INT
  EXEC @rc = asisp_createRMFProfileDefEx  @profile_id = @profile_id,
                                          @metric_basename = @metric_basename,
                                          @param_01 = @param_01,
                                          @param_02 = @param_02,
                                          @param_03 = @param_03,
                                          @param_04 = NULL,
                                          @param_05 = NULL,
                                          @metric_comparison = @metric_comparison,
                                          @metric_value = @metric_value,
                                          @metric_id = @metric_id,
                                          @metric_category = @metric_category,
                                          @metric_description = @metric_description,
                                          @bEnabled = @bEnabled
  RETURN @rc
GO

---------------------------------------------------------------------------------
PRINT 'Done.'
GO

