use Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='U' AND name='GEM_APMGenericMsgs')
  DROP TABLE GEM_APMGenericMsgs
GO

IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id(N'[dbo].[gemsp_processAPMGenericMsg]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[gemsp_processAPMGenericMsg]
GO

SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

CREATE PROCEDURE [gemsp_processAPMGenericMsg] 
		@tbsmclass  NVARCHAR(128)='',  -- contain product;version
		@instanceid NVARCHAR(128)=''  ,
		@instancename NVARCHAR(128)=''  ,
		@probetype NVARCHAR(128)=''  ,
		@probeid NVARCHAR(128)=''  ,
		@probename NVARCHAR(128)=''  ,
		@probeargs NVARCHAR(128)='', 
		@severity NVARCHAR(128)='', 
		@probevalue NVARCHAR(128)='', 
		@probecondition NVARCHAR(128)='', 
		@msg NVARCHAR(255)='',
		@eehost NVARCHAR(128)='',
		@hostos INT,
		@tcphostname NVARCHAR(128)='',
		@tmr NVARCHAR(128)='', 
                @hostnameslot NVARCHAR(128)='',
		@data1 NVARCHAR(128)='',
		@data2 NVARCHAR(128)='',
		@data3 NVARCHAR(128)='',
		@data4 NVARCHAR(128)=''                		
AS

RAISERROR('gemsp_processAPMGenericMsg: @tbsmclass=%s, @instanceid=%s, @instancename=%s, 
	  @probetype=%s, @probeid=%s, @probename=%s, @probeargs=%s, @severity=%s, 
	  @probevalue=%s, @probecondition=%s, @msg=%s, @eehost=%s, @hostos=%d, @tcphostname=%s,
	  @tmr=%s, @hostnameslot=%s, @data1=%s, @data2=%s, @data3=%s, @data4=%s', 0, 1,        		
 	  @tbsmclass, @instanceid, @instancename, @probetype, @probeid, @probename, @probeargs, 
	  @severity, @probevalue, @probecondition, @msg, @eehost, @hostos, @tcphostname, 
  	  @tmr, @hostnameslot, @data1, @data2, @data3, @data4)

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN gemsp_processAPMGenericMsg
ELSE
  SAVE TRAN gemsp_processAPMGenericMsg

DECLARE @h INT, @error NVARCHAR(128)
DECLARE @deleteproc NVARCHAR(16)
DECLARE @componenttype NVARCHAR(64)


CREATE TABLE #thresholdinstances(
	cid	ClassID,
	iid	ObjID)


EXEC gemsp_print 'gemsp_processAPMGenericMsg begins...'

DECLARE @checkslotchanges TINYINT 
DECLARE @instancecreated TINYINT
DECLARE @msglocal NVARCHAR(128)
SELECT @msglocal = 'no further details available'
DECLARE @goodsofar INT
SET @goodsofar = 1
DECLARE @disposition NVARCHAR(128)

DECLARE @print_string NVARCHAR(1024)

SET @checkslotchanges = 1
SET @instancecreated = 0 


IF @probeid IS NULL OR @probeid = ''
  SET @probeid = @probename

--- jpf concern about length of msg should we check the length AND TRUNCATE
IF @probeid IS NULL OR @probeid = ''
  SET @probeid = @msg


IF @probeid IS NULL OR @probeid = '' BEGIN
  SET @disposition = 'This event doesn''t seem to be a REAL threshold. We''ll ignore it'
  goto GETOUT
END 


IF @msg IS NULL OR @msg = '' BEGIN
  SET @msg = '(No description provided)'
END 


-- this event has a probeid so it IS a REAL threshold

DECLARE @placementhostname NVARCHAR(128)
DECLARE @oproduct NVARCHAR(128)
DECLARE @oversion NVARCHAR(128)

SET @placementhostname = @tcphostname


DECLARE @productName NVARCHAR(64)
DECLARE @version NVARCHAR(64)
DECLARE @manufacturer NVARCHAR(64)

DECLARE @cid ClassID
DECLARE @oid ObjID
DECLARE @holdrowcount INT

-- BEGIN the process to pull out product AND version FROM input @tbsmclass 
DECLARE @mpv_expr NVARCHAR(64)   /*pulls out pv */
SET @mpv_expr = '\(.*\);\(.*\);'  

IF SUBSTRING(@tbsmclass, len(@tbsmclass), 1) <> ';'
  SET @tbsmclass = @tbsmclass + ';'

EXEC @h = master..xp_regexmatch @mpv_expr, @tbsmclass , @h OUT, @error OUT
IF @h IS NULL BEGIN
     SET @disposition =  ' Failed to parse product/version ...  regular expression match failed' 
     SET @goodsofar = 0 
    END
ELSE BEGIN
    DECLARE @c INT, @i INT
    EXEC master..xp_regexmatcharraycount @h, @c OUT
      IF @c = 3	BEGIN
		 	EXEC master..xp_regexmatcharrayitem @h, 1, @oproduct OUTPUT
			EXEC master..xp_regexmatcharrayitem @h, 2, @oversion OUTPUT
		END 
      ELSE BEGIN
             SET @disposition =  'Failed to parse product/version ... COUNT of tokens IS NOT three'
             SET @goodsofar = 0 
      END 
      EXEC master..xp_regexmatcharrayfree @h
      SET @h=NULL  -- reset to NULL 

END -- END IF found product version with regular expression check

IF @goodsofar = 0 BEGIN
     goto GETOUT
END 

---- END of pulled out product AND version 

--- jpf need to get out IF goodsofar IS NOT 1 

-- get the CID 

SELECT @componenttype = comptype, @cid = cid, @manufacturer = manufacturer FROM GEMLookupCID WHERE prodname = @oproduct AND version = @oversion
IF @cid IS NOT NULL BEGIN
  SET @print_string = ' product name IS ' + @oproduct + ' version IS ' + @oversion + 'cid = ' + @cid
  EXEC gemsp_print @print_string
END
ELSE BEGIN
     SET @disposition =  ' cid NOT found product name IS ' + @oproduct + ' version IS ' + @oversion
     goto GETOUT
END 
---  END get CID

IF @componenttype <> 'generic' SET @checkslotchanges=0

--- now get instance id FOR searches AND creating instance
--- jpf need to ADD IN the hostname slot FOR something ELSE just IN CASE 

DECLARE @displayname NVARCHAR(256)

IF @instanceid IS NULL BEGIN
       SELECT @disposition = 'No Instance ID. Event IS invalid'
       goto GETOUT
END

SET @displayname = @instancename

IF @displayname IS NULL OR @displayname = ''
  SET @displayname = @instanceid

IF @placementhostname IS NOT NULL SET @displayname = @displayname + '@' +  @placementhostname

-- now search FOR the instance IN the IID TABLE
SET @holdrowcount = 0
IF @componenttype = 'generic' BEGIN 
-- this key only makes sense IF component IS generic
   INSERT #thresholdinstances
   SELECT DISTINCT cid=CID, oid=IID FROM GEM_IDLookup
       WHERE CID = @cid
       AND ((key_name = 'KeyGen' AND key_value like @instanceid))

   SET @holdrowcount=@@ROWCOUNT
END 
ELSE 
--- IF it IS NOT a generic event THEN we are matching ON hostnameslot OR tcpipaddress
---     AS the last resort
   IF @hostnameslot IS NOT NULL AND @hostnameslot <> '' BEGIN
      DECLARE @keytofind NVARCHAR(255)
      SET @keytofind = @hostnameslot

      IF @tmr IS NOT NULL SET @keytofind = @keytofind + '|' + @tmr 
      ELSE SET @keytofind = @keytofind + '|%' 

      INSERT #thresholdinstances
      SELECT DISTINCT cid=CID, oid=IID FROM GEM_IDLookup
          WHERE CID = @cid
          AND ((key_name = 'KeyTivName' AND key_value like @keytofind))

       SET @holdrowcount=@@ROWCOUNT
   END

ELSE 
  IF @tcphostname IS NOT NULL AND @tcphostname <> '' BEGIN 
      INSERT #thresholdinstances
      SELECT DISTINCT cid=CID, oid=IID FROM GEM_IDLookup
          WHERE CID = @cid
          AND ((key_name = 'KeyIp' AND key_value like @tcphostname))

       SET @holdrowcount=@@ROWCOUNT
   END  
  
---
-- get the oid
SELECT @oid = iid FROM #thresholdinstances

--- first try to handle the CASE WHERE the tcpipaddress changes AND the resource needs to 
--- be deleted AND THEN rerun through the CREATE process 

IF @holdrowcount > 0 AND @componenttype = 'generic'  BEGIN 
      DECLARE @temp NVARCHAR(255)
      SET @temp = NULL
      EXEC gemsp_getAttribute @cid, @oid, '_TCPHost', @temp OUTPUT

      IF ((@temp IS NOT NULL) AND (@tcphostname IS NULL)) OR 
            ((@temp IS NULL) AND (@tcphostname IS NOT NULL)) OR 
               (@tcphostname <> @temp) 
      BEGIN

        -- send the GONE FOR this oid
         EXEC gemsp_print 'deleting oid since the tcpiphostname changed'
         SELECT @deleteproc = 'delete_' + @cid
         EXEC @deleteproc @oid
        -- now make the rest of the code process this heartbeat AS a first time heartbeat
         SET @oid = NULL
         SET @holdrowcount = 0  --- 
        --- at this point the rest of the code will think that the resource does NOT exist

      END

END


--- no instance at this point

IF @holdrowcount = 0 BEGIN
          EXEC gemsp_print 'did NOT find matches'   

          IF @probetype = 'GONE'BEGIN
             SELECT @disposition = 'Gone message received, no instance created'
             goto GETOUT
          END 

          
          IF @componenttype <> 'generic' BEGIN 
             SELECT @disposition = 'waiting FOR instance to be discovered'
             goto GETOUT
          END 


		  -- no instances but cid EXISTS ... CREATE a instance
          DECLARE @parentid ObjID
          DECLARE @parentclass ClassID

          -- find OR find/CREATE the parent container
          EXEC gemsp_get_parent @cid, @tcphostname, @placementhostname, @hostos, @eehost, 
                                @parentid OUTPUT, @parentclass OUTPUT, @msglocal OUTPUT
         /* we can CREATE the instance since we don't expect a heartbeat FROM normal instrumentation */
         IF @parentid > 0 BEGIN  -- success finding OR creating  parent object 
           DECLARE @timestamp DATETIME
           SELECT @timestamp = GETDATE()
           DECLARE @createproc NVARCHAR(16)
           SELECT @createproc = 'asisp_create' + @cid
           SET @checkslotchanges = 0
           SET @instancecreated = 1
           
           DECLARE @pcid ClassID
           SELECT @pcid=base_cid FROM isa WHERE cid=@cid
           
           IF @pcid = 'GMGN'
             EXEC @oid = @createproc @displayname,
                                     @parentclass,
                                     @parentid,
                                     @instanceid,
                                     @displayname,  -- previous dispname
                                     @eehost,
                                     @tmr,
                                     0,          -- no heartbeat interval
                                     @timestamp,
                                     @manufacturer,
                                     @oproduct,
                                     @tcphostname,
                                     @oversion
           ELSE IF @pcid = 'GMGX' 
             EXEC @oid = @createproc @displayname,
                                     @parentclass,
                                     @parentid,
                                     @data1,
                                     @data2,
                                     @data3,
                                     @data4,
                                     @hostnameslot,
                                     @instanceid,
                                     @displayname,  -- previous dispname
                                     @eehost,
                                     @tmr,
                                     0,          -- no heartbeat interval
                                     @timestamp,
                                     @manufacturer,
                                     @oproduct,
                                     @tcphostname,
                                     @oversion                                 

             IF @oid > 0 BEGIN  --- CREATE worked
                 SET @disposition = 'created cid ' + @cid + ' oid ' + CAST(@oid AS NVARCHAR(10))
                 EXEC gemsp_print @disposition
                 DECLARE @lob_disp NVARCHAR(128)
                 SET @lob_disp = NULL
                 EXEC gemsp_LOBs @cid, @oid, @lob_disp OUTPUT -- put the instance IN a line of business
                 IF @lob_disp IS NOT NULL SET @disposition = @disposition + ', ' + @lob_disp

                 /* we want to cache the new oid so we can pick it up quickly NEXT time the threshold comes IN. */
               --- UPDATE lookup TABLE
                 INSERT INTO GEM_IDLookup (CID, IID, key_name, key_value) 
                   VALUES (@cid, @oid, 'KeyGen', @instanceid)
                END -- created the instance
              ELSE BEGIN -- CREATE failed 
                   RAISERROR('TBSM2007E: %s could not create object', 0, 1, @createproc) with log
                   SET @disposition = 'TBSM2007E: ' + @createproc + ' could NOT CREATE object'
		   SET @goodsofar = 0 
                   END
           END  --- END of success creating parent object 
          ELSE BEGIN  -- failed to find OR CREATE parent object 
            RAISERROR('TBSM2008E: error creating %s object: %s', 0, 1, @cid, @msglocal) with log
            SET @disposition = 'TBSM2008E: error creating ' + @cid + ' object: ' + @msglocal
            SET @goodsofar = 0 
          END

END  --- END of no instances found with initial lookup 

IF @goodsofar = 0 BEGIN
     goto GETOUT
END 

IF @holdrowcount = 0 BEGIN 
    INSERT #thresholdinstances
     SELECT DISTINCT cid=CID, oid=IID FROM GEM_IDLookup
            WHERE CID = @cid
               AND ((key_name = 'KeyGen' AND key_value like @instanceid))
     SET @holdrowcount = @@ROWCOUNT
END 

IF @holdrowcount > 0 AND @probetype = 'GONE' BEGIN
      SELECT @disposition = 'deleted ' + @cid + ' oid ' + CAST(@oid AS NVARCHAR(10))
      SELECT @deleteproc = 'delete_' + @cid
      EXEC @deleteproc @oid
        -- now make the rest of the code process this heartbeat AS a first time heartbeat
      SET @oid = NULL
      SET @holdrowcount = 0
END 

IF @holdrowcount > 0 BEGIN


-- CREATE the exception AND get out 
-- getExceptionType will CREATE the exception object FOR this software component IF necessary
   EXEC asisp_getExceptionType @cid, @cid, @probeid, 1, 5, NULL, 'GEM', 'Dynamically defined' 
   SET @disposition = @disposition + ', processed cid ' + @cid + ': '
-- process each instance 
   DECLARE iCursor CURSOR FOR 
   SELECT cid, iid FROM #thresholdinstances
   OPEN iCursor
   SELECT * FROM #thresholdinstances
   FETCH NEXT FROM iCursor INTO @cid, @oid
   WHILE @@FETCH_STATUS = 0 BEGIN
      SET @print_string = 'top of loop, cid=' + @cid + ', oid=' + CAST(@oid AS NVARCHAR(6))
      EXEC gemsp_print @print_string
      SET @disposition = @disposition + CAST(@oid AS NVARCHAR(10)) + ' '
      -- remove existing exception FOR this threshold IF necessary 
      DECLARE @currState INT
      DECLARE @currPriority INT
      DECLARE @time DATETIME

      /* SELECT @currState=_AlertState, @currPriority=_Priority FROM Exception_V 
           WHERE name=@probeid AND _ParentCID=@cid AND _ParentID=@oid AND deleted=0

      IF @currState IS NOT NULL  BEGIN
         SELECT @time = CURRENT_TIMESTAMP
         EXEC asisp_createEXCP @name=@probeid, @parent_cid=@cid, @parent_id=@oid, @DATE=@time,
                            @TIME=@time, @EXCP_NAME=@probeid, @EXCP_CD='OK', @AlertState=@currState, @Priority=@currPriority
      END 
      -- defect#: 69855 */
      
      SELECT @time = CURRENT_TIMESTAMP
      EXEC asisp_createEXCP @name=@probeid, @parent_cid=@cid, @parent_id=@oid, @DATE=@time,
                            @TIME=@time, @EXCP_NAME=@probeid, @EXCP_CD='OK', @AlertState=-1, @Priority=-1      
  
    -- CREATE new exception FOR the threshold, reflecting the current severity
      -- get priority	
      SELECT @currState=AlertState, @currPriority=Priority FROM GEMSeverities WHERE Severity = @severity
      IF @currState IS NULL --- USE DEFAULT state AND priority IF incoming IS NOT recognized  jpf need to SET disposition here
          SELECT @currState=AlertState, @currPriority=Priority FROM GEMSeverities WHERE Severity = 'INFORMATIONAL'

      -- SPECIAL CASE STATE MONITOR 
      -- IF this IS the state monitor, we'll force it to always be propagated.  It should only have a Red OR Green
      -- alert state.  IF it's Red, it overrides all other thresholds.  IF it's Green, thresholds will override it.

      IF @probeid = 'State' OR @probeid = 'STATE' SET @currPriority = 1 -- "critical priority" IS always propagated up



      SELECT @time = CURRENT_TIMESTAMP

--- jpf check what the probe type IS FOR Msg AND exceptions
      IF @probetype IS NULL  OR @probetype NOT like 'MSG'  BEGIN
          
 EXEC asisp_createEXCP @name=@probeid, @parent_cid=@cid, @parent_id=@oid, @DATE=@time, @TIME=@time, 
                            @EXCP_NAME=@probeid, @EXCP_CD=@msg, 
                            @AlertState=@currState, @Priority=@currPriority
      END
      ELSE BEGIN
           SET @currPriority = 1 -- "critical priority" IS always propagated up
           EXEC asisp_createMESG
              @name=@probeid,
              @parent_cid=@cid,
              @parent_id=@oid,
              @DATE=@time,
              @TIME=@time,
              @MESSAGE=@probeid,
              @REASON_CD=@msg,
              @AlertState=@currState,
              @Priority=@currPriority
      END 

      FETCH NEXT FROM iCursor INTO @cid, @oid
    END
  CLOSE iCursor
  DEALLOCATE iCursor
   --- END of creating exception

END 


IF @checkslotchanges = 1 AND @oid IS NOT NULL BEGIN

    DECLARE @previousDispname NVARCHAR(128)
    DECLARE @currentName NVARCHAR(128)
    DECLARE @cmd NVARCHAR(1024)
    DECLARE @cname NVARCHAR(128)
    DECLARE @attrTable NVARCHAR(64)



  -- has the instance's display name changed ON the GEM screen?
    EXEC gemsp_getAttribute @cid, @oid, 'name', @currentName OUTPUT
    EXEC gemsp_getAttribute @cid, @oid, '_PreviousDisplayName', @previousDispname OUTPUT

    -- has the instance's display name changed ON the GEM screen?

    IF @displayname <> @previousDispname BEGIN

       --- start to build the UPDATE sql statement 
       SELECT @cname = cname FROM Meta.dbo.obj_class_table WHERE cid = @cid
       SELECT @attrTable = @cname + '_C'
       --- prefix the command ON just IN CASE part of the command was already built   jpf 
       SELECT @cmd = 'UPDATE ' + @attrTable + ' SET _LastHbTimestamp = ''' + CAST(GETDATE() AS NVARCHAR(128)) + ''' ' 

      SELECT @cmd = @cmd + ', _PreviousDisplayName = ''' + @displayname + ''''

      -- new GEM name only takes effect IF the instance name hasn't been changed ON the TBSM console

      IF @previousDispname = @currentName
        SELECT @cmd = @cmd + ', name = ''' + @displayname + ''''



    END


END 



--- log the event AND the disposition AND THEN get out 
GETOUT:

IF @disposition IS NULL OR @disposition = ''
  SELECT @disposition = 'ignored'

--- DROP TABLE  GEM_APMGenericMsgs
 IF NOT EXISTS (SELECT * FROM sysobjects WHERE type ='U' AND name = 'GEM_APMGenericMsgs')
      CREATE TABLE GEM_APMGenericMsgs
        (
		class  NVARCHAR(128) NULL,
		instanceid NVARCHAR(128) NULL  ,
		instancename NVARCHAR(128) NULL  ,
		probetype NVARCHAR(128) NULL ,
		probeid NVARCHAR(128) NULL ,
		probename NVARCHAR(128) NULL  ,
		probeargs NVARCHAR(128) NULL, 
		severity NVARCHAR(128) NULL, 
		probevalue NVARCHAR(128) NULL, 
		probecondition NVARCHAR(128) NULL, 
		msg NVARCHAR(255) NULL,
		eehost NVARCHAR(128) NULL,
		hostos INT,
		tcphostname NVARCHAR(128) NULL,
		tmr NVARCHAR(128) NULL,
                mgedsysname NVARCHAR(128) NULL,
                data1 NVARCHAR(128) NULL, 
                data2 NVARCHAR(128) NULL, 
                data3 NVARCHAR(128) NULL, 
                data4 NVARCHAR(128) NULL, 
	        disposition NVARCHAR(128) NULL,
		time DATETIME NULL)

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = 'GEM_Options')
  IF EXISTS (SELECT * FROM GEM_Options WHERE _Option = 'trace events' AND _Value = 'yes') BEGIN
    INSERT INTO GEM_APMGenericMsgs (class, instanceid, instancename, probetype, probeid, probename, probeargs, severity, probevalue, probecondition, msg, eehost, hostos, tcphostname, tmr, mgedsysname, data1, data2, data3, data4, disposition, time )
      VALUES (	@tbsmclass, 	
	  @instanceid, 
	  @instancename, 
	  @probetype, 
	  @probeid, 
	  @probename, 
	  @probeargs, 
	  @severity, 
	  @probevalue, 
	  @probecondition, 
	  @msg, 
	  @eehost, 
	  @hostos, 
	  @tcphostname, 
	  @tmr,
          @hostnameslot,
          @data1,
          @data2,
          @data3,
          @data4,
	  @disposition, 
	GETDATE()
		)
END 

-- commit transaction
IF @tcount = 0 
  COMMIT TRAN gemsp_processAPMGenericMsg

GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

-- ***************************************************************************

IF EXISTS (SELECT * FROM sysobjects WHERE type='U' AND name='GEM_APMTHRMsgs')
  DROP TABLE GEM_APMTHRMsgs
GO

SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

/****** Object:  Stored PROCEDURE dbo.gemsp_processAPMThresholdMsg    Script Date: 8/2/00 8:36:36 AM ******/
IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id(N'[dbo].[gemsp_processAPMThresholdMsg]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[gemsp_processAPMThresholdMsg]
GO



CREATE PROCEDURE [gemsp_processAPMThresholdMsg] 
		@class  NVARCHAR(128)='',
		@hostname_slot  NVARCHAR(128)='',
		@origin NVARCHAR(255)='',
		@suborigin NVARCHAR(255)='',
		@source NVARCHAR(128)='',
		@subsource NVARCHAR(128)='',
		@adapterhost  NVARCHAR(128)='' ,
		@probeid NVARCHAR(128)=''  ,
		@probeargs NVARCHAR(128)='', 
		@probevalue NVARCHAR(255)='', 
		@probelabel NVARCHAR(128)='', 
		@severity NVARCHAR(128)='', 
		@mgednode NVARCHAR(128)='',
                @eehost NVARCHAR(128)='',
                @ostype INT=-1,
                @endpointtcphostname NVARCHAR(128)='',
                @endpointtmr NVARCHAR(128)=''
	
AS

RAISERROR('gemsp_processAPMThresholdMsg: @class=%s, @hostname_slot=%s, @origin=%s, @suborigin=%s,
	  @source=%s, @subsource=%s, @adapterhost=%s, @probeid=%s, @probeargs=%s, @probevalue=%s,
	  @probelabel=%s, @severity=%s, @mgednode=%s, @eehost=%s, @ostype=%d, 
	  @endpointtcphostname=%s, @endpointtmr=%s', 0, 1, @class, @hostname_slot, @origin, 
	  @suborigin, @source, @subsource, @adapterhost, @probeid, @probeargs, @probevalue, 
	  @probelabel, @severity, @mgednode, @eehost, @ostype, @endpointtcphostname,
	  @endpointtmr)

-- Standard transaction logic
DECLARE @tcount INT
SELECT @tcount = @@TRANCOUNT
IF @tcount = 0
  BEGIN TRAN gemsp_processAPMThresholdMsg
ELSE
  SAVE TRAN gemsp_processAPMThresholdMsg

DECLARE @print_string NVARCHAR(1024)

EXEC gemsp_print 'gemsp_processAPMThresholdMsg begins...'

IF @class = 'APM_SYS_MONITOR' BEGIN
EXEC gemsp_print 'System Monitor Event Received ... NOT supported.'
goto getout
END 

SET @print_string = 'ostype=' + CAST(@ostype AS NVARCHAR)
EXEC gemsp_print @print_string
SET @print_string = 'endpointtmr=' + @endpointtmr
EXEC gemsp_print @print_string
SET @print_string = 'endpointtcphostname=' + @endpointtcphostname
EXEC gemsp_print @print_string



-- DECLARE @eehost NVARCHAR(128)
-- SET @eehost = 'ntmrs02.rtp.lab.tivoli.com'

DECLARE @msg NVARCHAR(128)
SELECT @msg = 'no further details available'

DECLARE @comptype NVARCHAR(32)

DECLARE @disposition NVARCHAR(128)
SELECT @disposition = 'ignored'

IF @probeid IS NULL OR @probeid = ''
  SET @probeid = @probelabel

IF @probeid IS NULL OR @probeid = ''
  EXEC gemsp_print 'This event doesn''t seem to be a REAL threshold. We''ll ignore it'

ELSE BEGIN -- this event has a probeid so it IS a REAL threshold

DECLARE @hostname NVARCHAR(128)

IF @adapterhost IS NOT NULL
  IF @adapterhost <> ''
    SET @hostname = @adapterhost

IF @hostname IS NULL
  IF @hostname_slot IS NOT NULL
    IF @hostname_slot <> ''
      SET @hostname = @hostname_slot

IF @endpointtcphostname IS NULL OR @endpointtcphostname  = '' 
  SET   @endpointtcphostname =  @hostname

DECLARE @productName NVARCHAR(64)
DECLARE @version NVARCHAR(64)
DECLARE @manufacturer NVARCHAR(64)

DECLARE @cid ClassID
DECLARE @oid ObjID

SET @print_string = 'gemsp_Threshold_to_IID input: @subsource=' + @subsource + ', origin=' + @origin + 
      ', suborigin=' + @suborigin + ', hostname_slot=' + @hostname_slot + ', adapterhost=' + @adapterhost
EXEC gemsp_print @print_string

-- sp_helptext gemsp_Threshold_to_IID

CREATE TABLE #thresholdinstances(
	cid	ClassID,
	iid	ObjID)

EXEC gemsp_Threshold_to_IID @origin, @suborigin, @subsource, @hostname_slot, 
                            @endpointtmr, @endpointtcphostname, @hostname_slot, @adapterhost

DECLARE @count INT
SELECT @count = COUNT(*) FROM #thresholdinstances
SET @print_string = CAST(@count AS NVARCHAR(4)) + ' threshold/instance combinations returned by gemsp_Threshold_to_IID'
EXEC gemsp_print @print_string

IF @count = 0 BEGIN
  -- Some DM monitors cannot be associated with a software component based 
  -- only the slots IN the threshold event. The subsource IN the event will 
  -- contain the DM Profile name which we can try to associate with a CID 
  -- using another TABLE the customer sets up.  We might even CREATE
  -- an instance of the CID IF we don't find an instance AND we don't expect
  -- a heartbeat FOR the software component.

  SELECT @cid = cid, @productName = prodname, @version = version, @manufacturer = manufacturer FROM GEMLookupCID 
         WHERE comptype = 'dm' AND cid IN (SELECT CID FROM GEM_DMtoCID WHERE profileName = @subsource)

  IF @cid IS NOT NULL BEGIN
    SET @comptype = 'dm'
    SET @print_string = 'GEM_DMtoCID maps ' + @subsource + ' to ' + @cid
    EXEC gemsp_print @print_string

    DECLARE @parentid ObjID
    DECLARE @parentclass ClassID

    IF @hostname_slot IS NULL OR @hostname_slot =''  BEGIN 
      EXEC gemsp_print 'This event cannot be used to CREATE a DM instance. Hostname slot IS NOT SET.'
    END 
    ELSE BEGIN


      -- find OR find/CREATE the parent container

      EXEC gemsp_get_parent @cid, @endpointtcphostname, '', @ostype, @eehost, 
                          @parentid OUTPUT, @parentclass OUTPUT, @msg OUTPUT

      /* we can CREATE the instance since we don't expect a heartbeat FROM normal instrumentation */

      IF @parentid > 0 BEGIN
        DECLARE @timestamp DATETIME
        SELECT @timestamp = GETDATE()

        DECLARE @createproc NVARCHAR(16)
        SELECT @createproc = 'asisp_create' + @cid

        EXEC @oid = @createproc @hostname_slot, 
                                 @parentclass,
                                 @parentid,
                                 @hostname,
                                 @origin,
                                 @source,
                                 @suborigin,
                                 @subsource,
                                 @eehost,
                                 @endpointtmr,
                                 0,          -- no heartbeat interval
                                 @timestamp,
                                 @manufacturer,
                                 @productName,
                                 @endpointtcphostname,
                                 @version

        IF @oid > 0 BEGIN 
          SET @disposition = 'created cid ' + @cid + ' oid ' + CAST(@oid AS NVARCHAR(10))
          EXEC gemsp_print @disposition

          DECLARE @lob_disp NVARCHAR(128)
          SET @lob_disp = NULL
          EXEC gemsp_LOBs @cid, @oid, @lob_disp OUTPUT -- put the instance IN a line of business
          IF @lob_disp IS NOT NULL SET @disposition = @disposition + ', ' + @lob_disp

          /* we want to cache the new oid so we can pick it up quickly NEXT time the threshold comes IN. */
          DECLARE @new_origin NVARCHAR(255)
          DECLARE @new_suborigin NVARCHAR(255)
          SET @new_origin = '1=' + @origin
          SET @new_suborigin = '0=EPObjName=' + @hostname
  
          EXEC gemsp_updateLookup @cid, @oid, @origin, @suborigin, 
                                NULL, @endpointtmr, @endpointtcphostname, @hostname_slot, NULL

          EXEC gemsp_Threshold_to_IID @origin, @suborigin, @subsource, @hostname_slot, 
                                    @endpointtmr, @endpointtcphostname, @hostname_slot, @adapterhost

          SELECT @count = COUNT(*) FROM #thresholdinstances

        END -- created the instance
        ELSE BEGIN
          RAISERROR('TBSM2007E: %s could NOT CREATE object', 0, 1, @createproc) with log
          SET @disposition = 'TBSM2007E: ' + @createproc + ' could NOT CREATE object'
        END
      END /* able to CREATE a new instance */
     ELSE BEGIN
      RAISERROR('TBSM2008E: error creating %s object: %s', 0, 1, @cid, @msg) with log
      SET @disposition = 'TBSM2008E: error creating ' + @cid + ' object: ' + @msg
      END

    END -- got the DM profile to map to a CID

   END -- hostname slot IS NOT SET 
  ELSE BEGIN  -- IF cid IS NULL 
    RAISERROR('TBSM3002W: %s IS NOT associated with a cid IN GEM_DMtoCID', 0, 1, @subsource)
    SET @disposition = 'TBSM3002W: ' + @subsource + ' IS NOT associated with a cid IN GEM_DMtoCID'
  END
END -- see IF the event can be associated with a CID via the DM profile name

IF @count > 0 BEGIN
  DECLARE @probename NVARCHAR(255)
  SET @probename = @probelabel
  IF @probeargs IS NOT NULL
    IF @probeargs <> '' SET @probename = @probename + ' ' + @probeargs

  /* the staged event loader will clog up IF we try to CREATE an exception with a NULL value
     FOR the EXCP_CD - S/390 thresholds are like this so we have to make the probevalue more
     pleasing */

  IF @probevalue IS NULL
    SET @probevalue = ''

  -- getExceptionType will CREATE the exception object FOR this software component IF necessary
  SELECT @cid = cid FROM #thresholdinstances
  EXEC asisp_getExceptionType @cid, @cid, @probename, 1, 5, NULL, 'GEM', 'Dynamically defined' 

  IF @disposition = 'ignored'
    SET @disposition = 'processed cid ' + @cid + ': '
  ELSE
    SET @disposition = @disposition + ', processed cid ' + @cid + ': '

  -- process each instance 
  DECLARE iCursor CURSOR FOR 
    SELECT cid, iid FROM #thresholdinstances
  OPEN iCursor
  FETCH NEXT FROM iCursor INTO @cid, @oid
  WHILE @@FETCH_STATUS = 0 BEGIN
    SET @print_string = 'top of loop, cid=' + @cid + ', oid=' + CAST(@oid AS NVARCHAR(6))
    EXEC gemsp_print @print_string
    SET @disposition = @disposition + CAST(@oid AS NVARCHAR(10)) + ' '

    DECLARE @currState INT
    DECLARE @currPriority INT
    DECLARE @time DATETIME

    DECLARE @stateMonitor INT
    SET @stateMonitor = 0
    IF @probeid like '[Ss][Tt][Aa][Tt][Ee]' OR @probelabel like '[Ss][Tt][Aa][Tt][Ee]' 
      SET @stateMonitor = 1

    IF @stateMonitor = 0 BEGIN
      -- remove existing exception FOR this threshold IF necessary
    
      /* SELECT @currState=_AlertState, @currPriority=_Priority FROM Exception_V 
             WHERE _EXCP_NAME=@probename AND _ParentCID=@cid AND _ParentID=@oid AND deleted=0
      IF @currState IS NOT NULL 
      BEGIN
        EXEC gemsp_print 'removing previous exception'
        SELECT @time = CURRENT_TIMESTAMP
        EXEC asisp_createEXCP @name=@probename, @parent_cid=@cid, @parent_id=@oid, @DATE=@time,
                              @TIME=@time, @EXCP_NAME=@probename, @EXCP_CD='OK', @AlertState=@currState, @Priority=@currPriority
      END
      ELSE EXEC gemsp_print 'no previous exception to remove'
      -- defect#: 69855 */
      
      SELECT @time = CURRENT_TIMESTAMP
      EXEC asisp_createEXCP @name=@probename, @parent_cid=@cid, @parent_id=@oid, @DATE=@time,
                            @TIME=@time, @EXCP_NAME=@probename, @EXCP_CD='OK', @AlertState=-1, @Priority=-1      
    END

    -- CREATE new exception FOR the threshold, reflecting the current severity

    SELECT @currState=AlertState, @currPriority=Priority FROM GEMSeverities WHERE Severity = @severity
    IF @currState IS NULL
      SELECT @currState=AlertState, @currPriority=Priority FROM GEMSeverities WHERE Severity = 'INFORMATIONAL'

    -- IF this IS the state monitor, we'll force it to always be propagated.  It should only have a Red OR Green
    -- alert state.  IF it's Red, it overrides all other thresholds.  IF it's Green, thresholds will override it.


    SELECT @time = CURRENT_TIMESTAMP

    IF @stateMonitor = 0 BEGIN
      EXEC gemsp_print 'setting current exception'
      EXEC asisp_createEXCP @name=@probename, @parent_cid=@cid, @parent_id=@oid, @DATE=@time, @TIME=@time, 
                            @EXCP_NAME=@probename, @EXCP_CD=@probevalue, 
                            @AlertState=@currState, @Priority=@currPriority
    END
    ELSE BEGIN
      EXEC gemsp_print 'creating exception FOR state monitor'
      SET @currPriority = 1 -- "critical priority" IS always propagated up
      EXEC asisp_createMESG
              @name=@probename,
              @parent_cid=@cid,
              @parent_id=@oid,
              @DATE=@time,
              @TIME=@time,
              @MESSAGE=@probename,
              @REASON_CD=@probevalue,
              @AlertState=@currState,
              @Priority=@currPriority
    END

    FETCH NEXT FROM iCursor INTO @cid, @oid
  END
  CLOSE iCursor
  DEALLOCATE iCursor
END
ELSE IF (@comptype <> 'dm') BEGIN
  RAISERROR('TBSM3003W: Received threshold FOR an instance we don''t have a heartbeat FOR: subsource=%s, origin=%s, suborigin=%s, hostname=%s', 
            0, 1, @subsource, @origin, @suborigin, @hostname_slot)
  SET @disposition = 'TBSM3003W: Received threshold FOR an instance we don''t have a heartbeat FOR'
END

END -- this event has a probeid so it IS a REAL threshold

IF NOT EXISTS (SELECT * FROM sysobjects WHERE type ='U' AND name = 'GEM_APMTHRMsgs')
   CREATE TABLE GEM_APMTHRMsgs
          (class  NVARCHAR(128) NULL,
           hostname  NVARCHAR(128) NULL,
           origin NVARCHAR(255) NULL,
           suborigin NVARCHAR(255) NULL,
           source NVARCHAR(128) NULL,
           subsource NVARCHAR(128) NULL,
           adapterhostname  NVARCHAR(128) NULL,
           probeid NVARCHAR(128) NULL,
           probeargs NVARCHAR(128) NULL, 
           probevalue NVARCHAR(255) NULL,
           probelabel NVARCHAR(128) NULL,
           severity NVARCHAR(128) NULL,
           mgednode NVARCHAR(128) NULL,
           eehost NVARCHAR(128) NULL,
           ostype INT NULL,
           endpointtmr NVARCHAR(128) NULL,
           endpointtcphostname NVARCHAR(128) NULL,
           disposition NVARCHAR(128) NULL,
           time DATETIME NULL)

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = 'GEM_Options')
  IF EXISTS (SELECT * FROM GEM_Options WHERE _Option = 'trace events' AND _Value = 'yes') BEGIN

    INSERT INTO GEM_APMTHRMsgs (class, hostname, origin, suborigin, source, subsource, adapterhostname, 
                                probeid, probeargs, probevalue, probelabel, severity,  mgednode, eehost, 
                                ostype, endpointtmr, endpointtcphostname, disposition, time)
      VALUES (@class, 
	@hostname_slot,
	@origin,
	@suborigin,
	@source,
	@subsource,
	@adapterhost,
	@probeid,
	@probeargs, 
	@probevalue, 
	@probelabel, 
	@severity, 
	@mgednode,
	@eehost,
	@ostype,
        @endpointtmr,
        @endpointtcphostname,
	@disposition,
	GETDATE()
		)
  END

getout: 
-- commit transaction
IF @tcount = 0 
  COMMIT TRAN gemsp_processAPMThresholdMsg

GO

SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON 
GO

